{
v. 7.1 - September 2014

Unit to perform calculations with real numbers in multi-precision.
Copyright (C) 2014 by Franco Milani <nadir@solaris.polarhome.com>

This unit is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License version 2.1
as published by the Free Software Foundation.

This unit is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program. If not, you may read the License on the GNU site
http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
}

{$packrecords 1}
UNIT mpcalc71;

INTERFACE

CONST

maxdigits = 5120;
maxbuffer = 10496;

VAR

usrdigits: word;
xstatus: byte;

TYPE

bufnum = array [1..maxbuffer] of byte;

extnum = record
s : byte;    {sign: 0 -, 1 +}
n : word;    {total number of digits: n=i+d}
i : word;    {digit number of the integer part: i=n-d}
d : word;    {digit number of the decimal part: d=n-i}
c : bufnum   {array of digits}
end;

{* summary of error codes reported by the byte xstatus *}
{
0  no error
1  error converting an extended to extnum
2  error converting an extnum to extended
3  overflow converting radians to degrees
4  overflow in square
5  overflow in addition
6  overflow in subtraction
7  overflow in multiplication
8  overflow in division
9  division by zero
10 square root of negative number
11 overflow in tangent
12 overflow in cotangent
13 overflow in secant
14 overflow in cosecant
15 arcsine argument < -1 or > 1
16 arccosine argument < -1 or > 1
17 arcsecant argument > -1 and < 1
18 arccosecant argument > -1 and < 1
19 overflow in exponential
20 natural logarithm argument <= 0
21 base 10 logarithm argument <= 0
22 incorrect operand in power
23 overflow in power
24 incorrect operand in power with longint exponent
25 overflow in power with longint exponent
26 overflow in approximation
27 overflow in hyperbolic sine
28 overflow in hyperbolic cosine
29 hyperbolic cotangent of zero
30 hyperbolic cosecant of zero
31 inverse hyperbolic cosine argument < 1
32 inverse hyperbolic tangent argument <= -1 or >= 1
33 inverse hyperbolic cotangent argument >= -1 and <= 1
34 inverse hyperbolic secant argument <= 0 or > 1
35 inverse hyperbolic cosecant of zero
36 incorrect argument in factorial
37 overflow in factorial
38 incorrect argument in double factorial
39 overflow in double factorial
50 argument <=-1 or >=1 in inverse error function xerfinv
51 overflow in inverse error function xerfinv
52 argument <=0 or >=2 in inverse complementary error function xerfcinv
53 overflow in inverse complementary error function xerfcinv
54 negative argument in cumulative normal distribution xphi
55 argument <0 or >=1/2 in inverse cumulative normal distribution xiphi
56 overflow in inverse cumulative normal distribution xiphi
57 argument <=0 or >=1 in inverse lower tail xilnor of the normal distribution
58 overflow in inverse lower tail xilnor of the normal distribution
59 argument <=0 or >=1 in inverse upper tail xiunor of the normal distribution
60 overflow in inverse upper tail xiunor of the normal distribution
}

{* procedures and functions *}

procedure xtox(e1: extnum; var e2: extnum);
{the extnum e1 is copied in e2}

procedure xelz(var e: extnum);
{eliminates superfluous zeroes}

procedure xsetn(n: word);
{to set the digit number}

procedure xdisplay(e: extnum);
{displays an extnum}

procedure xapp(e1: extnum; var e2: extnum; w: word);
{approximates e1 at the digit number w and stores the result in e2.
If e1.i>w, xstatus=26 and e2:=0}

procedure xtostr(e: extnum; var s: ansistring);
{converts an extnum to an ansistring}

procedure strtox(s: ansistring; var e: extnum; var w: word);
{converts an ansistring to an extnum. w contains the position of the
first error, or 0 if no error}

procedure pitox(var e: extnum);
{stores the constant pi on e}

procedure eutox(var e: extnum);
{stores the Euler's number on e}

procedure zerotox(var e: extnum);
{stores zero on e}

procedure onetox(var e: extnum);
{stores one on e}

procedure monetox(var e: extnum);
{stores minus one on e}

procedure twotox(var e: extnum);
{stores two on e}

procedure etox(ex: extended; var e: extnum);
{converts an extended to extnum. If error, xstatus=1}

procedure xtoe(e: extnum; var ex: extended);
{converts an extnum to extended.If error, xstatus=2}

procedure wtox(k: word; var e: extnum);
{converts a word to an extnum}

procedure ltox(l: longint; var e: extnum);
{converts a longint to an extnum}

procedure dwtox(c: longword; var e: extnum);
{converts a longword to an extnum}

procedure qwtox(q: qword; var e: extnum);
{converts a qword to an extnum }

function xeq(e1,e2: extnum): boolean;
{true if e1=e2, false otherwise}

function xne(e1,e2: extnum): boolean;
{true if e1<>e2, false if e1=e2}

function xgt(e1,e2: extnum): boolean;
{true if e1>e2, false otherwise}

function xlt(e1,e2: extnum): boolean;
{true if e1<e2, false otherwise}

function xgte(e1,e2: extnum): boolean;
{true if e1>=e2, false otherwise}

function xlte(e1,e2: extnum): boolean;
{true if e1<=e2, false otherwise}

procedure xfrac(e1: extnum; var e2: extnum);
{the fractional part of e1 is stored in e2}

procedure xint(e1: extnum; var e2: extnum);
{the integer part of e1 is stored in e2}

function xsign(e: extnum): longint;
{sign function: -1 if e<0, 0 if e=0, 1 if e>0}

procedure xceil(e1: extnum; var e2: extnum);
{ceil function: the smallest integer >= e1 is stored in e2}

procedure xfloor(e1: extnum; var e2: extnum);
{floor function: the largest integer <= e1 is stored in e2 }

procedure xabs(e1: extnum; var e2: extnum);
{absolute value: e2:=|e1|}

procedure xadd(e1,e2: extnum; var e3: extnum);
{addition: e3:=e1+e2. If overflow, xstatus=5}

procedure xsub(e1,e2: extnum; var e3: extnum);
{subtraction: e3:=e1-e2. If overflow, xstatus=6}

procedure xmul(e1,e2: extnum; var e3: extnum);
{multiplication: e3:=e1*e2. If overflow, xstatus=7}

procedure xdiv(e1,e2: extnum; var e3: extnum);
{division: e3:=e1/e2. If overflow, xstatus=8. If e2=0 (division by zero),
xstatus=9}

procedure xmulq(e1: extnum; q: qword; var e2:extnum);
{multiplication by a qword q: e2:=e1*q}

procedure xdivq(e1: extnum; q: qword; var e2: extnum);
{division by a qword q: e2:=e1/q}

procedure xmul2(e1: extnum; var e2: extnum); {multiplication by 2}

procedure xmul2s(var e: extnum);             {multiplication by 2}

procedure xdiv2(e1: extnum; var e2: extnum); {division by 2}

procedure xdiv2s(var e: extnum);             {division by 2}

procedure xinv(e1: extnum; var e2: extnum);
{the reciprocal of e1 is stored in e2. If overflow, xstatus=8. If e2=0
(division by zero), xstatus=9}

procedure xpow(e1,e2: extnum; var e3: extnum);
{power e3:=e1^e2. If the operand values are incorrect, i.e. if e1=0 and e2<=0
or e1<0 and e2 not integer, xstatus=22. If overflow, xstatus=23}

procedure xpow1(e1: extnum; k: longint; var e2: extnum);
{power e3:=e1^k. If the operand values are incorrect, i.e. if e1=0 and k<=0,
xstatus=24. If overflow, xstatus=25}

procedure xsqr(e1: extnum; var e2: extnum);
{square e2:=e1^2. If overflow, xstatus=4}

procedure xsqrt(e1: extnum; var e2: extnum);
{square root: e2:=sqrt(e1). If e2<0, xstatus=10}

procedure xfact(e1: extnum; var e2: extnum);
{factorial: e2:=e1!. If e1 is not integer or e1<0, xstatus=36.
If overflow, xstatus=37}

procedure xdfact(e1: extnum; var e2: extnum);
{double factorial: e2:=e1!!=1*3*5*...*e1. If e1 is not integer or not odd
or e1<0, xstatus=38. If overflow, xstatus=39}

procedure xdtor(e1: extnum; var e2: extnum);
{converts degrees to radians}

procedure xrtod(e1: extnum; var e2: extnum);
{converts radians to degrees. If overflow, xstatus=3}

procedure xexp(e1: extnum; var e2: extnum);
{exponential: e2:=exp(e1). If overflow, xstatus=19}

procedure xln(e1: extnum; var e2: extnum);
{natural logarithm: e2:=ln(e1). If e1<=0, xstatus=20}

procedure xlog(e1: extnum; var e2: extnum);
{base 10 logarithm: e2:=log(e1). If e1<=0, xstatus=21}

{* trigonometric functions, argument in radians *}

procedure xsin(e1: extnum; var e2: extnum);
{trigonometric sine: e2:=sin(e1)}

procedure xcos(e1: extnum; var e2: extnum);
{trigonometric cosine: e2:=cos(e1)}

procedure xtan(e1: extnum; var e2: extnum);
{trigonometric tangent: e2:=tan(e1). If overflow, xstatus=11}

procedure xcot(e1: extnum; var e2: extnum);
{trigonometric cotangent: e2:=cot(e1). If overflow, xstatus=12}

procedure xsec(e1: extnum; var e2: extnum);
{trigonometric secant: e2:=sec(e1). If overflow, xstatus=13}

procedure xcsc(e1: extnum; var e2: extnum);
{trigonometric cosecant: e2:=csc(e1). If overflow, xstatus=14}

{* inverse trigonometric functions, in radians *}

procedure xasin(e1: extnum; var e2: extnum);
{inverse trigonometric sine: e2:=arcsin(e1). If e1<-1 or e1>1, xstatus=15}

procedure xacos(e1: extnum; var e2: extnum);
{inverse trigonometric cosine: e2:=arccos(e1). If e1<-1 or e1>1, xstatus=16}

procedure xatan(e1: extnum; var e2: extnum);
{inverse trigonometric tangent: e2:=arctan(e1)}

procedure xacot(e1: extnum; var e2: extnum);
{inverse trigonometric cotangent: e2:=arccot(e1)}

procedure xasec(e1: extnum; var e2: extnum);
{inverse trigonometric secant: e2:=arcsec(e1). If e1>-1 and e1<1, xstatus=17}

procedure xacsc(e1: extnum; var e2: extnum);
{inverse trigonometric cosecant: e2:=arccsc(e1). If e1>-1 and e1<1, xstatus=18}

{* hyperbolic functions *}

procedure xsinh(e1: extnum; var e2: extnum);
{hyperbolic sine: e2:=sinh(e1). If overflow, xstatus=27}

procedure xcosh(e1: extnum; var e2: extnum);
{hyperbolic cosine: e2:=cosh(e1). If overflow, xstatus=28}

procedure xtanh(e1: extnum; var e2: extnum);
{hyperbolic tangent: e2:=tanh(e1)}

procedure xcoth(e1: extnum; var e2: extnum);
{hyperbolic cotangent: e2:=coth(e1). If e1=0, xstatus=29}

procedure xsech(e1: extnum; var e2: extnum);
{hyperbolic secant: e2:=sech(e1)}

procedure xcsch(e1: extnum; var e2: extnum);
{hyperbolic cosecant: e2:=csch(e1). If e1=0, xstatus=30}

{* inverse hyperbolic functions *}

procedure xasinh(e1: extnum; var e2: extnum);
{inverse hyperbolic sine: e2:=arsinh(e1)}

procedure xacosh(e1: extnum; var e2: extnum);
{inverse hyperbolic cosine: e2:=arcosh(e1). If e1<1, xstatus=31}

procedure xatanh(e1: extnum; var e2: extnum);
{inverse hyperbolic tangent: e2:=artanh(e1). If e1<=-1 or e1>=1, xstatus=32}

procedure xacoth(e1: extnum; var e2: extnum);
{inverse hyperbolic cotangent: e2:=arcoth(e1). If e1>=-1 and e1<=1, xstatus=33}

procedure xasech(e1: extnum; var e2: extnum);
{inverse hyperbolic secant: e2:=arsech(e1). If e1<=0 or e1>1, xstatus=34}

procedure xacsch(e1: extnum; var e2: extnum);
{inverse hyperbolic cosecant: e2:=arcsch(e1). If e1=0, xstatus=35}

{* bitwise functions calculated on the integer part, integer results *}

procedure andm(e1,e2: extnum; var e3: extnum);
{e3:=e1 and e2}

procedure orm(e1,e2: extnum; var e3: extnum);
{e3:=e1 or e2}

procedure xorm(e1,e2: extnum; var e3: extnum);
{e3:=e1 xor e2}

procedure notm(e1: extnum; var e2: extnum);
{e2:=not(e1)}

{* special functions *}

procedure xerf(e1: extnum; var e2: extnum);
{Error function: e2:=xerf(e1). If overflow, returns +1 if x>0 or -1 if x<0}

procedure xerfinv(e1: extnum; var e2: extnum);
{Inverse error function: e2:=xerfinv(e1). If x<=-1 or x>=+1 (out of domain),
xstatus=50; if overflow, xstatus=51}

procedure xerfc(e1: extnum; var e2: extnum);
{Complementary error function: e2:=xerfc(e1)=1-xerf(e1). If overflow, returns
0 or 2}

procedure xerfcinv(e1: extnum; var e2: extnum);
{inverse complementary error function: e2:=xerfcinv(e1). If e1<=0 or e1>=2
(out of domain), xstatus=52; if overflow, xstatus=53}

procedure xphi(e1: extnum; var e2: extnum);
{cumulative normal distribution function: 1/sqrt(2*pi) multiplied by
the integral from 0 to x of exp(-t^2/2)dt. xphi(x)=0.5*xerf(x/sqrt(2)).
If e1<0 (out of domain), xstatus=54. If overflow, returns 1/2}

procedure xiphi(e1: extnum; var e2: extnum);
{inverse cumulative normal distribution function: e2:=xiphi(e1).
If e1<0 or e1>=1/2 (out of domain), xstatus=55; if overflow, xstatus=56}

procedure xlnor(e1: extnum; var e2: extnum);
{lower tail cumulative probability of the standard normal distribution:
1/sqrt(2*pi) multiplied by the integral from -inf to x of exp(-t^2/2)dt.
xlnor(x)=0.5*(1+xerf(x/sqrt(2))). If overflow, returns 0 or 1}

procedure xilnor(e1: extnum; var e2: extnum);
{inverse lower tail cumulative probability of the standard normal distribution:
e2:=xilnor(e1). If e1<=0 or e1>=1 (out of domain), xstatus=57; if overflow,
xstatus=58}

procedure xunor(e1: extnum; var e2: extnum);
{upper tail cumulative probability of the standard normal distribution:
1/sqrt(2*pi) multiplied by the integral from x to +inf of exp(-t^2/2)dt.
xunor(x)=0.5*(1-xerf(x/sqrt(2))). If overflow, returns 0 or 1}

procedure xiunor(e1: extnum; var e2: extnum);
{inverse upper tail cumulative probability of the standard normal distribution:
e2:=xiunor(e1). If x<=0 or x>=1 (out of domain), xstatus=59; if overflow,
xstatus=60}

IMPLEMENTATION

TYPE

nbuffer = array [0..600] of longword;
vbuffer = array [1..(maxbuffer div 8)+1] of qword;

CONST

{some numeric constants needed for the evaluation of the functions}
ar0    : array [1..8] of byte = (1,1,0,1,0,0,0,0);          {0}
arm1   : array [1..8] of byte = (0,1,0,1,0,0,0,1);          {-1}
ar1    : array [1..8] of byte = (1,1,0,1,0,0,0,1);          {1}
ar2    : array [1..8] of byte = (1,1,0,1,0,0,0,2);          {2}
ar3    : array [1..8] of byte = (1,1,0,1,0,0,0,3);          {3}
ar4    : array [1..8] of byte = (1,1,0,1,0,0,0,4);          {4}
ar6    : array [1..8] of byte = (1,1,0,1,0,0,0,6);          {6}
ar14   : array [1..9] of byte = (1,2,0,2,0,0,0,1,4);        {14}
ar24   : array [1..9] of byte = (1,2,0,2,0,0,0,2,4);        {24}
ar31   : array [1..9] of byte = (1,2,0,2,0,0,0,3,1);        {31}
ar48   : array [1..9] of byte = (1,2,0,2,0,0,0,4,8);        {48}
ar107  : array [1..10] of byte = (1,3,0,3,0,0,0,1,0,7);     {107}
ar180  : array [1..10] of byte = (1,3,0,3,0,0,0,1,8,0);     {180}
ar1489 : array [1..11] of byte = (1,4,0,4,0,0,0,1,4,8,9);   {1489}
ar0d25 : array [1..10] of byte = (1,3,0,1,0,2,0,0,2,5);     {0.25}
ar0d3  : array [1..9] of byte = (1,2,0,1,0,1,0,0,3);        {0.3}
ar0d447: array [1..11] of byte = (1,4,0,1,0,3,0,0,4,4,7);   {0.447}
ar0d5  : array [1..9] of byte = (1,2,0,1,0,1,0,0,5);        {0.5}
ar0d69 : array [1..10] of byte = (1,3,0,1,0,2,0,0,6,9);     {0.69}
ar0d75 : array [1..10] of byte = (1,3,0,1,0,2,0,0,7,5);     {0.75}
ar1d5  : array [1..9] of byte = (1,2,0,1,0,1,0,1,5);        {1.5}
ar1d75 : array [1..10] of byte = (1,3,0,1,0,2,0,1,7,5);     {1.75}
ar3d5  : array [1..9] of byte = (1,2,0,1,0,1,0,3,5);        {3.5}
ar4d25 : array [1..10] of byte = (1,3,0,1,0,2,0,4,2,5);     {4.25}
ar5d4  : array [1..9] of byte = (1,2,0,1,0,1,0,5,4);        {5.4}
ar13d5 : array [1..10] of byte = (1,3,0,2,0,1,0,1,3,5);     {13.5}
ar14d5 : array [1..10] of byte = (1,3,0,2,0,1,0,1,4,5);     {14.5}

vcc : qword = 100000000;

dpot : array[0..4] of word = (1,10,100,1000,10000);

d10 : array[0..99] of byte =
(0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,
3,3,4,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,
7,7,7,8,8,8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,9);

m10 : array[0..99] of byte =
(0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,
8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,
7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9);

ck : array[0..19] of byte  = (0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1);

cd : array[-10..9] of byte = (0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9);

cu : array[-10..9] of byte = (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0);

tr : array[0..99] of word  =
(0,1,1,1,2,2,2,2,2,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,
6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,
8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9);

p1 : longword = 10;
p2 : longword = 100;
p3 : longword = 1000;
p4 : longword = 10000;
p5 : longword = 100000;
p6 : longword = 1000000;
p7 : longword = 10000000;

xeu: array [1..5166] of byte =
(1,0,20,1,0,255,19,2,7,1,8,2,8,1,8,2,8,4,5,9,0,4,5,2,3,5,3,6,0,2,8,7,4,7,1,3,
 5,2,6,6,2,4,9,7,7,5,7,2,4,7,0,9,3,6,9,9,9,5,9,5,7,4,9,6,6,9,6,7,6,2,7,7,2,4,
 0,7,6,6,3,0,3,5,3,5,4,7,5,9,4,5,7,1,3,8,2,1,7,8,5,2,5,1,6,6,4,2,7,4,2,7,4,6,
 6,3,9,1,9,3,2,0,0,3,0,5,9,9,2,1,8,1,7,4,1,3,5,9,6,6,2,9,0,4,3,5,7,2,9,0,0,3,
 3,4,2,9,5,2,6,0,5,9,5,6,3,0,7,3,8,1,3,2,3,2,8,6,2,7,9,4,3,4,9,0,7,6,3,2,3,3,
 8,2,9,8,8,0,7,5,3,1,9,5,2,5,1,0,1,9,0,1,1,5,7,3,8,3,4,1,8,7,9,3,0,7,0,2,1,5,
 4,0,8,9,1,4,9,9,3,4,8,8,4,1,6,7,5,0,9,2,4,4,7,6,1,4,6,0,6,6,8,0,8,2,2,6,4,8,
 0,0,1,6,8,4,7,7,4,1,1,8,5,3,7,4,2,3,4,5,4,4,2,4,3,7,1,0,7,5,3,9,0,7,7,7,4,4,
 9,9,2,0,6,9,5,5,1,7,0,2,7,6,1,8,3,8,6,0,6,2,6,1,3,3,1,3,8,4,5,8,3,0,0,0,7,5,
 2,0,4,4,9,3,3,8,2,6,5,6,0,2,9,7,6,0,6,7,3,7,1,1,3,2,0,0,7,0,9,3,2,8,7,0,9,1,
 2,7,4,4,3,7,4,7,0,4,7,2,3,0,6,9,6,9,7,7,2,0,9,3,1,0,1,4,1,6,9,2,8,3,6,8,1,9,
 0,2,5,5,1,5,1,0,8,6,5,7,4,6,3,7,7,2,1,1,1,2,5,2,3,8,9,7,8,4,4,2,5,0,5,6,9,5,
 3,6,9,6,7,7,0,7,8,5,4,4,9,9,6,9,9,6,7,9,4,6,8,6,4,4,5,4,9,0,5,9,8,7,9,3,1,6,
 3,6,8,8,9,2,3,0,0,9,8,7,9,3,1,2,7,7,3,6,1,7,8,2,1,5,4,2,4,9,9,9,2,2,9,5,7,6,
 3,5,1,4,8,2,2,0,8,2,6,9,8,9,5,1,9,3,6,6,8,0,3,3,1,8,2,5,2,8,8,6,9,3,9,8,4,9,
 6,4,6,5,1,0,5,8,2,0,9,3,9,2,3,9,8,2,9,4,8,8,7,9,3,3,2,0,3,6,2,5,0,9,4,4,3,1,
 1,7,3,0,1,2,3,8,1,9,7,0,6,8,4,1,6,1,4,0,3,9,7,0,1,9,8,3,7,6,7,9,3,2,0,6,8,3,
 2,8,2,3,7,6,4,6,4,8,0,4,2,9,5,3,1,1,8,0,2,3,2,8,7,8,2,5,0,9,8,1,9,4,5,5,8,1,
 5,3,0,1,7,5,6,7,1,7,3,6,1,3,3,2,0,6,9,8,1,1,2,5,0,9,9,6,1,8,1,8,8,1,5,9,3,0,
 4,1,6,9,0,3,5,1,5,9,8,8,8,8,5,1,9,3,4,5,8,0,7,2,7,3,8,6,6,7,3,8,5,8,9,4,2,2,
 8,7,9,2,2,8,4,9,9,8,9,2,0,8,6,8,0,5,8,2,5,7,4,9,2,7,9,6,1,0,4,8,4,1,9,8,4,4,
 4,3,6,3,4,6,3,2,4,4,9,6,8,4,8,7,5,6,0,2,3,3,6,2,4,8,2,7,0,4,1,9,7,8,6,2,3,2,
 0,9,0,0,2,1,6,0,9,9,0,2,3,5,3,0,4,3,6,9,9,4,1,8,4,9,1,4,6,3,1,4,0,9,3,4,3,1,
 7,3,8,1,4,3,6,4,0,5,4,6,2,5,3,1,5,2,0,9,6,1,8,3,6,9,0,8,8,8,7,0,7,0,1,6,7,6,
 8,3,9,6,4,2,4,3,7,8,1,4,0,5,9,2,7,1,4,5,6,3,5,4,9,0,6,1,3,0,3,1,0,7,2,0,8,5,
 1,0,3,8,3,7,5,0,5,1,0,1,1,5,7,4,7,7,0,4,1,7,1,8,9,8,6,1,0,6,8,7,3,9,6,9,6,5,
 5,2,1,2,6,7,1,5,4,6,8,8,9,5,7,0,3,5,0,3,5,4,0,2,1,2,3,4,0,7,8,4,9,8,1,9,3,3,
 4,3,2,1,0,6,8,1,7,0,1,2,1,0,0,5,6,2,7,8,8,0,2,3,5,1,9,3,0,3,3,2,2,4,7,4,5,0,
 1,5,8,5,3,9,0,4,7,3,0,4,1,9,9,5,7,7,7,7,0,9,3,5,0,3,6,6,0,4,1,6,9,9,7,3,2,9,
 7,2,5,0,8,8,6,8,7,6,9,6,6,4,0,3,5,5,5,7,0,7,1,6,2,2,6,8,4,4,7,1,6,2,5,6,0,7,
 9,8,8,2,6,5,1,7,8,7,1,3,4,1,9,5,1,2,4,6,6,5,2,0,1,0,3,0,5,9,2,1,2,3,6,6,7,7,
 1,9,4,3,2,5,2,7,8,6,7,5,3,9,8,5,5,8,9,4,4,8,9,6,9,7,0,9,6,4,0,9,7,5,4,5,9,1,
 8,5,6,9,5,6,3,8,0,2,3,6,3,7,0,1,6,2,1,1,2,0,4,7,7,4,2,7,2,2,8,3,6,4,8,9,6,1,
 3,4,2,2,5,1,6,4,4,5,0,7,8,1,8,2,4,4,2,3,5,2,9,4,8,6,3,6,3,7,2,1,4,1,7,4,0,2,
 3,8,8,9,3,4,4,1,2,4,7,9,6,3,5,7,4,3,7,0,2,6,3,7,5,5,2,9,4,4,4,8,3,3,7,9,9,8,
 0,1,6,1,2,5,4,9,2,2,7,8,5,0,9,2,5,7,7,8,2,5,6,2,0,9,2,6,2,2,6,4,8,3,2,6,2,7,
 7,9,3,3,3,8,6,5,6,6,4,8,1,6,2,7,7,2,5,1,6,4,0,1,9,1,0,5,9,0,0,4,9,1,6,4,4,9,
 9,8,2,8,9,3,1,5,0,5,6,6,0,4,7,2,5,8,0,2,7,7,8,6,3,1,8,6,4,1,5,5,1,9,5,6,5,3,
 2,4,4,2,5,8,6,9,8,2,9,4,6,9,5,9,3,0,8,0,1,9,1,5,2,9,8,7,2,1,1,7,2,5,5,6,3,4,
 7,5,4,6,3,9,6,4,4,7,9,1,0,1,4,5,9,0,4,0,9,0,5,8,6,2,9,8,4,9,6,7,9,1,2,8,7,4,
 0,6,8,7,0,5,0,4,8,9,5,8,5,8,6,7,1,7,4,7,9,8,5,4,6,6,7,7,5,7,5,7,3,2,0,5,6,8,
 1,2,8,8,4,5,9,2,0,5,4,1,3,3,4,0,5,3,9,2,2,0,0,0,1,1,3,7,8,6,3,0,0,9,4,5,5,6,
 0,6,8,8,1,6,6,7,4,0,0,1,6,9,8,4,2,0,5,5,8,0,4,0,3,3,6,3,7,9,5,3,7,6,4,5,2,0,
 3,0,4,0,2,4,3,2,2,5,6,6,1,3,5,2,7,8,3,6,9,5,1,1,7,7,8,8,3,8,6,3,8,7,4,4,3,9,
 6,6,2,5,3,2,2,4,9,8,5,0,6,5,4,9,9,5,8,8,6,2,3,4,2,8,1,8,9,9,7,0,7,7,3,3,2,7,
 6,1,7,1,7,8,3,9,2,8,0,3,4,9,4,6,5,0,1,4,3,4,5,5,8,8,9,7,0,7,1,9,4,2,5,8,6,3,
 9,8,7,7,2,7,5,4,7,1,0,9,6,2,9,5,3,7,4,1,5,2,1,1,1,5,1,3,6,8,3,5,0,6,2,7,5,2,
 6,0,2,3,2,6,4,8,4,7,2,8,7,0,3,9,2,0,7,6,4,3,1,0,0,5,9,5,8,4,1,1,6,6,1,2,0,5,
 4,5,2,9,7,0,3,0,2,3,6,4,7,2,5,4,9,2,9,6,6,6,9,3,8,1,1,5,1,3,7,3,2,2,7,5,3,6,
 4,5,0,9,8,8,8,9,0,3,1,3,6,0,2,0,5,7,2,4,8,1,7,6,5,8,5,1,1,8,0,6,3,0,3,6,4,4,
 2,8,1,2,3,1,4,9,6,5,5,0,7,0,4,7,5,1,0,2,5,4,4,6,5,0,1,1,7,2,7,2,1,1,5,5,5,1,
 9,4,8,6,6,8,5,0,8,0,0,3,6,8,5,3,2,2,8,1,8,3,1,5,2,1,9,6,0,0,3,7,3,5,6,2,5,2,
 7,9,4,4,9,5,1,5,8,2,8,4,1,8,8,2,9,4,7,8,7,6,1,0,8,5,2,6,3,9,8,1,3,9,5,5,9,9,
 0,0,6,7,3,7,6,4,8,2,9,2,2,4,4,3,7,5,2,8,7,1,8,4,6,2,4,5,7,8,0,3,6,1,9,2,9,8,
 1,9,7,1,3,9,9,1,4,7,5,6,4,4,8,8,2,6,2,6,0,3,9,0,3,3,8,1,4,4,1,8,2,3,2,6,2,5,
 1,5,0,9,7,4,8,2,7,9,8,7,7,7,9,9,6,4,3,7,3,0,8,9,9,7,0,3,8,8,8,6,7,7,8,2,2,7,
 1,3,8,3,6,0,5,7,7,2,9,7,8,8,2,4,1,2,5,6,1,1,9,0,7,1,7,6,6,3,9,4,6,5,0,7,0,6,
 3,3,0,4,5,2,7,9,5,4,6,6,1,8,5,5,0,9,6,6,6,6,1,8,5,6,6,4,7,0,9,7,1,1,3,4,4,4,
 7,4,0,1,6,0,7,0,4,6,2,6,2,1,5,6,8,0,7,1,7,4,8,1,8,7,7,8,4,4,3,7,1,4,3,6,9,8,
 8,2,1,8,5,5,9,6,7,0,9,5,9,1,0,2,5,9,6,8,6,2,0,0,2,3,5,3,7,1,8,5,8,8,7,4,8,5,
 6,9,6,5,2,2,0,0,0,5,0,3,1,1,7,3,4,3,9,2,0,7,3,2,1,1,3,9,0,8,0,3,2,9,3,6,3,4,
 4,7,9,7,2,7,3,5,5,9,5,5,2,7,7,3,4,9,0,7,1,7,8,3,7,9,3,4,2,1,6,3,7,0,1,2,0,5,
 0,0,5,4,5,1,3,2,6,3,8,3,5,4,4,0,0,0,1,8,6,3,2,3,9,9,1,4,9,0,7,0,5,4,7,9,7,7,
 8,0,5,6,6,9,7,8,5,3,3,5,8,0,4,8,9,6,6,9,0,6,2,9,5,1,1,9,4,3,2,4,7,3,0,9,9,5,
 8,7,6,5,5,2,3,6,8,1,2,8,5,9,0,4,1,3,8,3,2,4,1,1,6,0,7,2,2,6,0,2,9,9,8,3,3,0,
 5,3,5,3,7,0,8,7,6,1,3,8,9,3,9,6,3,9,1,7,7,9,5,7,4,5,4,0,1,6,1,3,7,2,2,3,6,1,
 8,7,8,9,3,6,5,2,6,0,5,3,8,1,5,5,8,4,1,5,8,7,1,8,6,9,2,5,5,3,8,6,0,6,1,6,4,7,
 7,9,8,3,4,0,2,5,4,3,5,1,2,8,4,3,9,6,1,2,9,4,6,0,3,5,2,9,1,3,3,2,5,9,4,2,7,9,
 4,9,0,4,3,3,7,2,9,9,0,8,5,7,3,1,5,8,0,2,9,0,9,5,8,6,3,1,3,8,2,6,8,3,2,9,1,4,
 7,7,1,1,6,3,9,6,3,3,7,0,9,2,4,0,0,3,1,6,8,9,4,5,8,6,3,6,0,6,0,6,4,5,8,4,5,9,
 2,5,1,2,6,9,9,4,6,5,5,7,2,4,8,3,9,1,8,6,5,6,4,2,0,9,7,5,2,6,8,5,0,8,2,3,0,7,
 5,4,4,2,5,4,5,9,9,3,7,6,9,1,7,0,4,1,9,7,7,7,8,0,0,8,5,3,6,2,7,3,0,9,4,1,7,1,
 0,1,6,3,4,3,4,9,0,7,6,9,6,4,2,3,7,2,2,2,9,4,3,5,2,3,6,6,1,2,5,5,7,2,5,0,8,8,
 1,4,7,7,9,2,2,3,1,5,1,9,7,4,7,7,8,0,6,0,5,6,9,6,7,2,5,3,8,0,1,7,1,8,0,7,7,6,
 3,6,0,3,4,6,2,4,5,9,2,7,8,7,7,8,4,6,5,8,5,0,6,5,6,0,5,0,7,8,0,8,4,4,2,1,1,5,
 2,9,6,9,7,5,2,1,8,9,0,8,7,4,0,1,9,6,6,0,9,0,6,6,5,1,8,0,3,5,1,6,5,0,1,7,9,2,
 5,0,4,6,1,9,5,0,1,3,6,6,5,8,5,4,3,6,6,3,2,7,1,2,5,4,9,6,3,9,9,0,8,5,4,9,1,4,
 4,2,0,0,0,1,4,5,7,4,7,6,0,8,1,9,3,0,2,2,1,2,0,6,6,0,2,4,3,3,0,0,9,6,4,1,2,7,
 0,4,8,9,4,3,9,0,3,9,7,1,7,7,1,9,5,1,8,0,6,9,9,0,8,6,9,9,8,6,0,6,6,3,6,5,8,3,
 2,3,2,2,7,8,7,0,9,3,7,6,5,0,2,2,6,0,1,4,9,2,9,1,0,1,1,5,1,7,1,7,7,6,3,5,9,4,
 4,6,0,2,0,2,3,2,4,9,3,0,0,2,8,0,4,0,1,8,6,7,7,2,3,9,1,0,2,8,8,0,9,7,8,6,6,6,
 0,5,6,5,1,1,8,3,2,6,0,0,4,3,6,8,8,5,0,8,8,1,7,1,5,7,2,3,8,6,6,9,8,4,2,2,4,2,
 2,0,1,0,2,4,9,5,0,5,5,1,8,8,1,6,9,4,8,0,3,2,2,1,0,0,2,5,1,5,4,2,6,4,9,4,6,3,
 9,8,1,2,8,7,3,6,7,7,6,5,8,9,2,7,6,8,8,1,6,3,5,9,8,3,1,2,4,7,7,8,8,6,5,2,0,1,
 4,1,1,7,4,1,1,0,9,1,3,6,0,1,1,6,4,9,9,5,0,7,6,6,2,9,0,7,7,9,4,3,6,4,6,0,0,5,
 8,5,1,9,4,1,9,9,8,5,6,0,1,6,2,6,4,7,9,0,7,6,1,5,3,2,1,0,3,8,7,2,7,5,5,7,1,2,
 6,9,9,2,5,1,8,2,7,5,6,8,7,9,8,9,3,0,2,7,6,1,7,6,1,1,4,6,1,6,2,5,4,9,3,5,6,4,
 9,5,9,0,3,7,9,8,0,4,5,8,3,8,1,8,2,3,2,3,3,6,8,6,1,2,0,1,6,2,4,3,7,3,6,5,6,9,
 8,4,6,7,0,3,7,8,5,8,5,3,3,0,5,2,7,5,8,3,3,3,3,7,9,3,9,9,0,7,5,2,1,6,6,0,6,9,
 2,3,8,0,5,3,3,6,9,8,8,7,9,5,6,5,1,3,7,2,8,5,5,9,3,8,8,3,4,9,9,8,9,4,7,0,7,4,
 1,6,1,8,1,5,5,0,1,2,5,3,9,7,0,6,4,6,4,8,1,7,1,9,4,6,7,0,8,3,4,8,1,9,7,2,1,4,
 4,8,8,8,9,8,7,9,0,6,7,6,5,0,3,7,9,5,9,0,3,6,6,9,6,7,2,4,9,4,9,9,2,5,4,5,2,7,
 9,0,3,3,7,2,9,6,3,6,1,6,2,6,5,8,9,7,6,0,3,9,4,9,8,5,7,6,7,4,1,3,9,7,3,5,9,4,
 4,1,0,2,3,7,4,4,3,2,9,7,0,9,3,5,5,4,7,7,9,8,2,6,2,9,6,1,4,5,9,1,4,4,2,9,3,6,
 4,5,1,4,2,8,6,1,7,1,5,8,5,8,7,3,3,9,7,4,6,7,9,1,8,9,7,5,7,1,2,1,1,9,5,6,1,8,
 7,3,8,5,7,8,3,6,4,4,7,5,8,4,4,8,4,2,3,5,5,5,5,8,1,0,5,0,0,2,5,6,1,1,4,9,2,3,
 9,1,5,1,8,8,9,3,0,9,9,4,6,3,4,2,8,4,1,3,9,3,6,0,8,0,3,8,3,0,9,1,6,6,2,8,1,8,
 8,1,1,5,0,3,7,1,5,2,8,4,9,6,7,0,5,9,7,4,1,6,2,5,6,2,8,2,3,6,0,9,2,1,6,8,0,7,
 5,1,5,0,1,7,7,7,2,5,3,8,7,4,0,2,5,6,4,2,5,3,4,7,0,8,7,9,0,8,9,1,3,7,2,9,1,7,
 2,2,8,2,8,6,1,1,5,1,5,9,1,5,6,8,3,7,2,5,2,4,1,6,3,0,7,7,2,2,5,4,4,0,6,3,3,7,
 8,7,5,9,3,1,0,5,9,8,2,6,7,6,0,9,4,4,2,0,3,2,6,1,9,2,4,2,8,5,3,1,7,0,1,8,7,8,
 1,7,7,2,9,6,0,2,3,5,4,1,3,0,6,0,6,7,2,1,3,6,0,4,6,0,0,0,3,8,9,6,6,1,0,9,3,6,
 4,7,0,9,5,1,4,1,4,1,7,1,8,5,7,7,7,0,1,4,1,8,0,6,0,6,4,4,3,6,3,6,8,1,5,4,6,4,
 4,4,0,0,5,3,3,1,6,0,8,7,7,8,3,1,4,3,1,7,4,4,4,0,8,1,1,9,4,9,4,2,2,9,7,5,5,9,
 9,3,1,4,0,1,1,8,8,8,6,8,3,3,1,4,8,3,2,8,0,2,7,0,6,5,5,3,8,3,3,0,0,4,6,9,3,2,
 9,0,1,1,5,7,4,4,1,4,7,5,6,3,1,3,9,9,9,7,2,2,1,7,0,3,8,0,4,6,1,7,0,9,2,8,9,4,
 5,7,9,0,9,6,2,7,1,6,6,2,2,6,0,7,4,0,7,1,8,7,4,9,9,7,5,3,5,9,2,1,2,7,5,6,0,8,
 4,4,1,4,7,3,7,8,2,3,3,0,3,2,7,0,3,3,0,1,6,8,2,3,7,1,9,3,6,4,8,0,0,2,1,7,3,2,
 8,5,7,3,4,9,3,5,9,4,7,5,6,4,3,3,4,1,2,9,9,4,3,0,2,4,8,5,0,2,3,5,7,3,2,2,1,4,
 5,9,7,8,4,3,2,8,2,6,4,1,4,2,1,6,8,4,8,7,8,7,2,1,6,7,3,3,6,7,0,1,0,6,1,5,0,9,
 4,2,4,3,4,5,6,9,8,4,4,0,1,8,7,3,3,1,2,8,1,0,1,0,7,9,4,5,1,2,7,2,2,3,7,3,7,8,
 8,6,1,2,6,0,5,8,1,6,5,6,6,8,0,5,3,7,1,4,3,9,6,1,2,7,8,8,8,7,3,2,5,2,7,3,7,3,
 8,9,0,3,9,2,8,9,0,5,0,6,8,6,5,3,2,4,1,3,8,0,6,2,7,9,6,0,2,5,9,3,0,3,8,7,7,2,
 7,6,9,7,7,8,3,7,9,2,8,6,8,4,0,9,3,2,5,3,6,5,8,8,0,7,3,3,9,8,8,4,5,7,2,1,8,7,
 4,6,0,2,1,0,0,5,3,1,1,4,8,3,3,5,1,3,2,3,8,5,0,0,4,7,8,2,7,1,6,9,3,7,6,2,1,8,
 0,0,4,9,0,4,7,9,5,5,9,7,9,5,9,2,9,0,5,9,1,6,5,5,4,7,0,5,0,5,7,7,7,5,1,4,3,0,
 8,1,7,5,1,1,2,6,9,8,9,8,5,1,8,8,4,0,8,7,1,8,5,6,4,0,2,6,0,3,5,3,0,5,5,8,3,7,
 3,7,8,3,2,4,2,2,9,2,4,1,8,5,6,2,5,6,4,4,2,5,5,0,2,2,6,7,2,1,5,5,9,8,0,2,7,4,
 0,1,2,6,1,7,9,7,1,9,2,8,0,4,7,1,3,9,6,0,0,6,8,9,1,6,3,8,2,8,6,6,5,2,7,7,0,0,
 9,7,5,2,7,6,7,0,6,9,7,7,7,0,3,6,4,3,9,2,6,0,2,2,4,3,7,2,8,4,1,8,4,0,8,8,3,2,
 5,1,8,4,8,7,7,0,4,7,2,6,3,8,4,4,0,3,7,9,5,3,0,1,6,6,9,0,5,4,6,5,9,3,7,4,6,1,
 6,1,9,3,2,3,8,4,0,3,6,3,8,9,3,1,3,1,3,6,4,3,2,7,1,3,7,6,8,8,8,4,1,0,2,6,8,1,
 1,2,1,9,8,9,1,2,7,5,2,2,3,0,5,6,2,5,6,7,5,6,2,5,4,7,0,1,7,2,5,0,8,6,3,4,9,7,
 6,5,3,6,7,2,8,8,6,0,5,9,6,6,7,5,2,7,4,0,8,6,8,6,2,7,4,0,7,9,1,2,8,5,6,5,7,6,
 9,9,6,3,1,3,7,8,9,7,5,3,0,3,4,6,6,0,6,1,6,6,6,9,8,0,4,2,1,8,2,6,7,7,2,4,5,6,
 0,5,3,0,6,6,0,7,7,3,8,9,9,6,2,4,2,1,8,3,4,0,8,5,9,8,8,2,0,7,1,8,6,4,6,8,2,6,
 2,3,2,1,5,0,8,0,2,8,8,2,8,6,3,5,9,7,4,6,8,3,9,6,5,4,3,5,8,8,5,6,6,8,5,5,0,3,
 7,7,3,1,3,1,2,9,6,5,8,7,9,7,5,8,1,0,5,0,1,2,1,4,9,1,6,2,0,7,6,5,6,7,6,9,9,5,
 0,6,5,9,7,1,5,3,4,4,7,6,3,4,7,0,3,2,0,8,5,3,2,1,5,6,0,3,6,7,4,8,2,8,6,0,8,3,
 7,8,6,5,6,8,0,3,0,7,3,0,6,2,6,5,7,6,3,3,4,6,9,7,7,4,2,9,5,6,3,4,6,4,3,7,1,6,
 7,0,9,3,9,7,1,9,3,0,6,0,8,7,6,9,6,3,4,9,5,3,2,8,8,4,6,8,3,3,6,1,3,0,3,8,8,2,
 9,4,3,1,0,4,0,8,0,0,2,9,6,8,7,3,8,6,9,1,1,7,0,6,6,6,6,6,1,4,6,8,0,0,0,1,5,1,
 2,1,1,4,3,4,4,2,2,5,6,0,2,3,8,7,4,4,7,4,3,2,5,2,5,0,7,6,9,3,8,7,0,7,7,7,7,5,
 1,9,3,2,9,9,9,4,2,1,3,7,2,7,7,2,1,1,2,5,8,8,4,3,6,0,8,7,1,5,8,3,4,8,3,5,6,2,
 6,9,6,1,6,6,1,9,8,0,5,7,2,5,2,6,6,1,2,2,0,6,7,9,7,5,4,0,6,2,1,0,6,2,0,8,0,6,
 4,9,8,8,2,9,1,8,4,5,4,3,9,5,3,0,1,5,2,9,9,8,2,0,9,2,5,0,3,0,0,5,4,9,8,2,5,7);

xpg: array [1..5166] of byte =
(1,0,20,1,0,255,19,3,1,4,1,5,9,2,6,5,3,5,8,9,7,9,3,2,3,8,4,6,2,6,4,3,3,8,3,2,
 7,9,5,0,2,8,8,4,1,9,7,1,6,9,3,9,9,3,7,5,1,0,5,8,2,0,9,7,4,9,4,4,5,9,2,3,0,7,
 8,1,6,4,0,6,2,8,6,2,0,8,9,9,8,6,2,8,0,3,4,8,2,5,3,4,2,1,1,7,0,6,7,9,8,2,1,4,
 8,0,8,6,5,1,3,2,8,2,3,0,6,6,4,7,0,9,3,8,4,4,6,0,9,5,5,0,5,8,2,2,3,1,7,2,5,3,
 5,9,4,0,8,1,2,8,4,8,1,1,1,7,4,5,0,2,8,4,1,0,2,7,0,1,9,3,8,5,2,1,1,0,5,5,5,9,
 6,4,4,6,2,2,9,4,8,9,5,4,9,3,0,3,8,1,9,6,4,4,2,8,8,1,0,9,7,5,6,6,5,9,3,3,4,4,
 6,1,2,8,4,7,5,6,4,8,2,3,3,7,8,6,7,8,3,1,6,5,2,7,1,2,0,1,9,0,9,1,4,5,6,4,8,5,
 6,6,9,2,3,4,6,0,3,4,8,6,1,0,4,5,4,3,2,6,6,4,8,2,1,3,3,9,3,6,0,7,2,6,0,2,4,9,
 1,4,1,2,7,3,7,2,4,5,8,7,0,0,6,6,0,6,3,1,5,5,8,8,1,7,4,8,8,1,5,2,0,9,2,0,9,6,
 2,8,2,9,2,5,4,0,9,1,7,1,5,3,6,4,3,6,7,8,9,2,5,9,0,3,6,0,0,1,1,3,3,0,5,3,0,5,
 4,8,8,2,0,4,6,6,5,2,1,3,8,4,1,4,6,9,5,1,9,4,1,5,1,1,6,0,9,4,3,3,0,5,7,2,7,0,
 3,6,5,7,5,9,5,9,1,9,5,3,0,9,2,1,8,6,1,1,7,3,8,1,9,3,2,6,1,1,7,9,3,1,0,5,1,1,
 8,5,4,8,0,7,4,4,6,2,3,7,9,9,6,2,7,4,9,5,6,7,3,5,1,8,8,5,7,5,2,7,2,4,8,9,1,2,
 2,7,9,3,8,1,8,3,0,1,1,9,4,9,1,2,9,8,3,3,6,7,3,3,6,2,4,4,0,6,5,6,6,4,3,0,8,6,
 0,2,1,3,9,4,9,4,6,3,9,5,2,2,4,7,3,7,1,9,0,7,0,2,1,7,9,8,6,0,9,4,3,7,0,2,7,7,
 0,5,3,9,2,1,7,1,7,6,2,9,3,1,7,6,7,5,2,3,8,4,6,7,4,8,1,8,4,6,7,6,6,9,4,0,5,1,
 3,2,0,0,0,5,6,8,1,2,7,1,4,5,2,6,3,5,6,0,8,2,7,7,8,5,7,7,1,3,4,2,7,5,7,7,8,9,
 6,0,9,1,7,3,6,3,7,1,7,8,7,2,1,4,6,8,4,4,0,9,0,1,2,2,4,9,5,3,4,3,0,1,4,6,5,4,
 9,5,8,5,3,7,1,0,5,0,7,9,2,2,7,9,6,8,9,2,5,8,9,2,3,5,4,2,0,1,9,9,5,6,1,1,2,1,
 2,9,0,2,1,9,6,0,8,6,4,0,3,4,4,1,8,1,5,9,8,1,3,6,2,9,7,7,4,7,7,1,3,0,9,9,6,0,
 5,1,8,7,0,7,2,1,1,3,4,9,9,9,9,9,9,8,3,7,2,9,7,8,0,4,9,9,5,1,0,5,9,7,3,1,7,3,
 2,8,1,6,0,9,6,3,1,8,5,9,5,0,2,4,4,5,9,4,5,5,3,4,6,9,0,8,3,0,2,6,4,2,5,2,2,3,
 0,8,2,5,3,3,4,4,6,8,5,0,3,5,2,6,1,9,3,1,1,8,8,1,7,1,0,1,0,0,0,3,1,3,7,8,3,8,
 7,5,2,8,8,6,5,8,7,5,3,3,2,0,8,3,8,1,4,2,0,6,1,7,1,7,7,6,6,9,1,4,7,3,0,3,5,9,
 8,2,5,3,4,9,0,4,2,8,7,5,5,4,6,8,7,3,1,1,5,9,5,6,2,8,6,3,8,8,2,3,5,3,7,8,7,5,
 9,3,7,5,1,9,5,7,7,8,1,8,5,7,7,8,0,5,3,2,1,7,1,2,2,6,8,0,6,6,1,3,0,0,1,9,2,7,
 8,7,6,6,1,1,1,9,5,9,0,9,2,1,6,4,2,0,1,9,8,9,3,8,0,9,5,2,5,7,2,0,1,0,6,5,4,8,
 5,8,6,3,2,7,8,8,6,5,9,3,6,1,5,3,3,8,1,8,2,7,9,6,8,2,3,0,3,0,1,9,5,2,0,3,5,3,
 0,1,8,5,2,9,6,8,9,9,5,7,7,3,6,2,2,5,9,9,4,1,3,8,9,1,2,4,9,7,2,1,7,7,5,2,8,3,
 4,7,9,1,3,1,5,1,5,5,7,4,8,5,7,2,4,2,4,5,4,1,5,0,6,9,5,9,5,0,8,2,9,5,3,3,1,1,
 6,8,6,1,7,2,7,8,5,5,8,8,9,0,7,5,0,9,8,3,8,1,7,5,4,6,3,7,4,6,4,9,3,9,3,1,9,2,
 5,5,0,6,0,4,0,0,9,2,7,7,0,1,6,7,1,1,3,9,0,0,9,8,4,8,8,2,4,0,1,2,8,5,8,3,6,1,
 6,0,3,5,6,3,7,0,7,6,6,0,1,0,4,7,1,0,1,8,1,9,4,2,9,5,5,5,9,6,1,9,8,9,4,6,7,6,
 7,8,3,7,4,4,9,4,4,8,2,5,5,3,7,9,7,7,4,7,2,6,8,4,7,1,0,4,0,4,7,5,3,4,6,4,6,2,
 0,8,0,4,6,6,8,4,2,5,9,0,6,9,4,9,1,2,9,3,3,1,3,6,7,7,0,2,8,9,8,9,1,5,2,1,0,4,
 7,5,2,1,6,2,0,5,6,9,6,6,0,2,4,0,5,8,0,3,8,1,5,0,1,9,3,5,1,1,2,5,3,3,8,2,4,3,
 0,0,3,5,5,8,7,6,4,0,2,4,7,4,9,6,4,7,3,2,6,3,9,1,4,1,9,9,2,7,2,6,0,4,2,6,9,9,
 2,2,7,9,6,7,8,2,3,5,4,7,8,1,6,3,6,0,0,9,3,4,1,7,2,1,6,4,1,2,1,9,9,2,4,5,8,6,
 3,1,5,0,3,0,2,8,6,1,8,2,9,7,4,5,5,5,7,0,6,7,4,9,8,3,8,5,0,5,4,9,4,5,8,8,5,8,
 6,9,2,6,9,9,5,6,9,0,9,2,7,2,1,0,7,9,7,5,0,9,3,0,2,9,5,5,3,2,1,1,6,5,3,4,4,9,
 8,7,2,0,2,7,5,5,9,6,0,2,3,6,4,8,0,6,6,5,4,9,9,1,1,9,8,8,1,8,3,4,7,9,7,7,5,3,
 5,6,6,3,6,9,8,0,7,4,2,6,5,4,2,5,2,7,8,6,2,5,5,1,8,1,8,4,1,7,5,7,4,6,7,2,8,9,
 0,9,7,7,7,7,2,7,9,3,8,0,0,0,8,1,6,4,7,0,6,0,0,1,6,1,4,5,2,4,9,1,9,2,1,7,3,2,
 1,7,2,1,4,7,7,2,3,5,0,1,4,1,4,4,1,9,7,3,5,6,8,5,4,8,1,6,1,3,6,1,1,5,7,3,5,2,
 5,5,2,1,3,3,4,7,5,7,4,1,8,4,9,4,6,8,4,3,8,5,2,3,3,2,3,9,0,7,3,9,4,1,4,3,3,3,
 4,5,4,7,7,6,2,4,1,6,8,6,2,5,1,8,9,8,3,5,6,9,4,8,5,5,6,2,0,9,9,2,1,9,2,2,2,1,
 8,4,2,7,2,5,5,0,2,5,4,2,5,6,8,8,7,6,7,1,7,9,0,4,9,4,6,0,1,6,5,3,4,6,6,8,0,4,
 9,8,8,6,2,7,2,3,2,7,9,1,7,8,6,0,8,5,7,8,4,3,8,3,8,2,7,9,6,7,9,7,6,6,8,1,4,5,
 4,1,0,0,9,5,3,8,8,3,7,8,6,3,6,0,9,5,0,6,8,0,0,6,4,2,2,5,1,2,5,2,0,5,1,1,7,3,
 9,2,9,8,4,8,9,6,0,8,4,1,2,8,4,8,8,6,2,6,9,4,5,6,0,4,2,4,1,9,6,5,2,8,5,0,2,2,
 2,1,0,6,6,1,1,8,6,3,0,6,7,4,4,2,7,8,6,2,2,0,3,9,1,9,4,9,4,5,0,4,7,1,2,3,7,1,
 3,7,8,6,9,6,0,9,5,6,3,6,4,3,7,1,9,1,7,2,8,7,4,6,7,7,6,4,6,5,7,5,7,3,9,6,2,4,
 1,3,8,9,0,8,6,5,8,3,2,6,4,5,9,9,5,8,1,3,3,9,0,4,7,8,0,2,7,5,9,0,0,9,9,4,6,5,
 7,6,4,0,7,8,9,5,1,2,6,9,4,6,8,3,9,8,3,5,2,5,9,5,7,0,9,8,2,5,8,2,2,6,2,0,5,2,
 2,4,8,9,4,0,7,7,2,6,7,1,9,4,7,8,2,6,8,4,8,2,6,0,1,4,7,6,9,9,0,9,0,2,6,4,0,1,
 3,6,3,9,4,4,3,7,4,5,5,3,0,5,0,6,8,2,0,3,4,9,6,2,5,2,4,5,1,7,4,9,3,9,9,6,5,1,
 4,3,1,4,2,9,8,0,9,1,9,0,6,5,9,2,5,0,9,3,7,2,2,1,6,9,6,4,6,1,5,1,5,7,0,9,8,5,
 8,3,8,7,4,1,0,5,9,7,8,8,5,9,5,9,7,7,2,9,7,5,4,9,8,9,3,0,1,6,1,7,5,3,9,2,8,4,
 6,8,1,3,8,2,6,8,6,8,3,8,6,8,9,4,2,7,7,4,1,5,5,9,9,1,8,5,5,9,2,5,2,4,5,9,5,3,
 9,5,9,4,3,1,0,4,9,9,7,2,5,2,4,6,8,0,8,4,5,9,8,7,2,7,3,6,4,4,6,9,5,8,4,8,6,5,
 3,8,3,6,7,3,6,2,2,2,6,2,6,0,9,9,1,2,4,6,0,8,0,5,1,2,4,3,8,8,4,3,9,0,4,5,1,2,
 4,4,1,3,6,5,4,9,7,6,2,7,8,0,7,9,7,7,1,5,6,9,1,4,3,5,9,9,7,7,0,0,1,2,9,6,1,6,
 0,8,9,4,4,1,6,9,4,8,6,8,5,5,5,8,4,8,4,0,6,3,5,3,4,2,2,0,7,2,2,2,5,8,2,8,4,8,
 8,6,4,8,1,5,8,4,5,6,0,2,8,5,0,6,0,1,6,8,4,2,7,3,9,4,5,2,2,6,7,4,6,7,6,7,8,8,
 9,5,2,5,2,1,3,8,5,2,2,5,4,9,9,5,4,6,6,6,7,2,7,8,2,3,9,8,6,4,5,6,5,9,6,1,1,6,
 3,5,4,8,8,6,2,3,0,5,7,7,4,5,6,4,9,8,0,3,5,5,9,3,6,3,4,5,6,8,1,7,4,3,2,4,1,1,
 2,5,1,5,0,7,6,0,6,9,4,7,9,4,5,1,0,9,6,5,9,6,0,9,4,0,2,5,2,2,8,8,7,9,7,1,0,8,
 9,3,1,4,5,6,6,9,1,3,6,8,6,7,2,2,8,7,4,8,9,4,0,5,6,0,1,0,1,5,0,3,3,0,8,6,1,7,
 9,2,8,6,8,0,9,2,0,8,7,4,7,6,0,9,1,7,8,2,4,9,3,8,5,8,9,0,0,9,7,1,4,9,0,9,6,7,
 5,9,8,5,2,6,1,3,6,5,5,4,9,7,8,1,8,9,3,1,2,9,7,8,4,8,2,1,6,8,2,9,9,8,9,4,8,7,
 2,2,6,5,8,8,0,4,8,5,7,5,6,4,0,1,4,2,7,0,4,7,7,5,5,5,1,3,2,3,7,9,6,4,1,4,5,1,
 5,2,3,7,4,6,2,3,4,3,6,4,5,4,2,8,5,8,4,4,4,7,9,5,2,6,5,8,6,7,8,2,1,0,5,1,1,4,
 1,3,5,4,7,3,5,7,3,9,5,2,3,1,1,3,4,2,7,1,6,6,1,0,2,1,3,5,9,6,9,5,3,6,2,3,1,4,
 4,2,9,5,2,4,8,4,9,3,7,1,8,7,1,1,0,1,4,5,7,6,5,4,0,3,5,9,0,2,7,9,9,3,4,4,0,3,
 7,4,2,0,0,7,3,1,0,5,7,8,5,3,9,0,6,2,1,9,8,3,8,7,4,4,7,8,0,8,4,7,8,4,8,9,6,8,
 3,3,2,1,4,4,5,7,1,3,8,6,8,7,5,1,9,4,3,5,0,6,4,3,0,2,1,8,4,5,3,1,9,1,0,4,8,4,
 8,1,0,0,5,3,7,0,6,1,4,6,8,0,6,7,4,9,1,9,2,7,8,1,9,1,1,9,7,9,3,9,9,5,2,0,6,1,
 4,1,9,6,6,3,4,2,8,7,5,4,4,4,0,6,4,3,7,4,5,1,2,3,7,1,8,1,9,2,1,7,9,9,9,8,3,9,
 1,0,1,5,9,1,9,5,6,1,8,1,4,6,7,5,1,4,2,6,9,1,2,3,9,7,4,8,9,4,0,9,0,7,1,8,6,4,
 9,4,2,3,1,9,6,1,5,6,7,9,4,5,2,0,8,0,9,5,1,4,6,5,5,0,2,2,5,2,3,1,6,0,3,8,8,1,
 9,3,0,1,4,2,0,9,3,7,6,2,1,3,7,8,5,5,9,5,6,6,3,8,9,3,7,7,8,7,0,8,3,0,3,9,0,6,
 9,7,9,2,0,7,7,3,4,6,7,2,2,1,8,2,5,6,2,5,9,9,6,6,1,5,0,1,4,2,1,5,0,3,0,6,8,0,
 3,8,4,4,7,7,3,4,5,4,9,2,0,2,6,0,5,4,1,4,6,6,5,9,2,5,2,0,1,4,9,7,4,4,2,8,5,0,
 7,3,2,5,1,8,6,6,6,0,0,2,1,3,2,4,3,4,0,8,8,1,9,0,7,1,0,4,8,6,3,3,1,7,3,4,6,4,
 9,6,5,1,4,5,3,9,0,5,7,9,6,2,6,8,5,6,1,0,0,5,5,0,8,1,0,6,6,5,8,7,9,6,9,9,8,1,
 6,3,5,7,4,7,3,6,3,8,4,0,5,2,5,7,1,4,5,9,1,0,2,8,9,7,0,6,4,1,4,0,1,1,0,9,7,1,
 2,0,6,2,8,0,4,3,9,0,3,9,7,5,9,5,1,5,6,7,7,1,5,7,7,0,0,4,2,0,3,3,7,8,6,9,9,3,
 6,0,0,7,2,3,0,5,5,8,7,6,3,1,7,6,3,5,9,4,2,1,8,7,3,1,2,5,1,4,7,1,2,0,5,3,2,9,
 2,8,1,9,1,8,2,6,1,8,6,1,2,5,8,6,7,3,2,1,5,7,9,1,9,8,4,1,4,8,4,8,8,2,9,1,6,4,
 4,7,0,6,0,9,5,7,5,2,7,0,6,9,5,7,2,2,0,9,1,7,5,6,7,1,1,6,7,2,2,9,1,0,9,8,1,6,
 9,0,9,1,5,2,8,0,1,7,3,5,0,6,7,1,2,7,4,8,5,8,3,2,2,2,8,7,1,8,3,5,2,0,9,3,5,3,
 9,6,5,7,2,5,1,2,1,0,8,3,5,7,9,1,5,1,3,6,9,8,8,2,0,9,1,4,4,4,2,1,0,0,6,7,5,1,
 0,3,3,4,6,7,1,1,0,3,1,4,1,2,6,7,1,1,1,3,6,9,9,0,8,6,5,8,5,1,6,3,9,8,3,1,5,0,
 1,9,7,0,1,6,5,1,5,1,1,6,8,5,1,7,1,4,3,7,6,5,7,6,1,8,3,5,1,5,5,6,5,0,8,8,4,9,
 0,9,9,8,9,8,5,9,9,8,2,3,8,7,3,4,5,5,2,8,3,3,1,6,3,5,5,0,7,6,4,7,9,1,8,5,3,5,
 8,9,3,2,2,6,1,8,5,4,8,9,6,3,2,1,3,2,9,3,3,0,8,9,8,5,7,0,6,4,2,0,4,6,7,5,2,5,
 9,0,7,0,9,1,5,4,8,1,4,1,6,5,4,9,8,5,9,4,6,1,6,3,7,1,8,0,2,7,0,9,8,1,9,9,4,3,
 0,9,9,2,4,4,8,8,9,5,7,5,7,1,2,8,2,8,9,0,5,9,2,3,2,3,3,2,6,0,9,7,2,9,9,7,1,2,
 0,8,4,4,3,3,5,7,3,2,6,5,4,8,9,3,8,2,3,9,1,1,9,3,2,5,9,7,4,6,3,6,6,7,3,0,5,8,
 3,6,0,4,1,4,2,8,1,3,8,8,3,0,3,2,0,3,8,2,4,9,0,3,7,5,8,9,8,5,2,4,3,7,4,4,1,7,
 0,2,9,1,3,2,7,6,5,6,1,8,0,9,3,7,7,3,4,4,4,0,3,0,7,0,7,4,6,9,2,1,1,2,0,1,9,1,
 3,0,2,0,3,3,0,3,8,0,1,9,7,6,2,1,1,0,1,1,0,0,4,4,9,2,9,3,2,1,5,1,6,0,8,4,2,4,
 4,4,8,5,9,6,3,7,6,6,9,8,3,8,9,5,2,2,8,6,8,4,7,8,3,1,2,3,5,5,2,6,5,8,2,1,3,1,
 4,4,9,5,7,6,8,5,7,2,6,2,4,3,3,4,4,1,8,9,3,0,3,9,6,8,6,4,2,6,2,4,3,4,1,0,7,7,
 3,2,2,6,9,7,8,0,2,8,0,7,3,1,8,9,1,5,4,4,1,1,0,1,0,4,4,6,8,2,3,2,5,2,7,1,6,2,
 0,1,0,5,2,6,5,2,2,7,2,1,1,1,6,6,0,3,9,6,6,6,5,5,7,3,0,9,2,5,4,7,1,1,0,5,5,7,
 8,5,3,7,6,3,4,6,6,8,2,0,6,5,3,1,0,9,8,9,6,5,2,6,9,1,8,6,2,0,5,6,4,7,6,9,3,1,
 2,5,7,0,5,8,6,3,5,6,6,2,0,1,8,5,5,8,1,0,0,7,2,9,3,6,0,6,5,9,8,7,6,4,8,6,1,1,
 7,9,1,0,4,5,3,3,4,8,8,5,0,3,4,6,1,1,3,6,5,7,6,8,6,7,5,3,2,4,9,4,4,1,6,6,8,0,
 3,9,6,2,6,5,7,9,7,8,7,7,1,8,5,5,6,0,8,4,5,5,2,9,6,5,4,1,2,6,6,5,4,0,8,5,3,0,
 6,1,4,3,4,4,4,3,1,8,5,8,6,7,6,9,7,5,1,4,5,6,6,1,4,0,6,8,0,0,7,0,0,2,3,7,8,7,
 7,6,5,9,1,3,4,4,0,1,7,1,2,7,4,9,4,7,0,4,2,0,5,6,2,2,3,0,5,3,8,9,9,4,5,6,1,3,
 1,4,0,7,1,1,2,7,0,0,0,4,0,7,8,5,4,7,3,3,2,6,9,9,3,9,0,8,1,4,5,4,6,6,4,6,4,5,
 8,8,0,7,9,7,2,7,0,8,2,6,6,8,3,0,6,3,4,3,2,8,5,8,7,8,5,6,9,8,3,0,5,2,3,5,8,0,
 8,9,3,3,0,6,5,7,5,7,4,0,6,7,9,5,4,5,7,1,6,3,7,7,5,2,5,4,2,0,2,1,1,4,9,5,5,7,
 6,1,5,8,1,4,0,0,2,5,0,1,2,6,2,2,8,5,9,4,1,3,0,2,1,6,4,7,1,5,5,0,9,7,9,2,5,9,
 2,3,0,9,9,0,7,9,6,5,4,7,3,7,6,1,2,5,5,1,7,6,5,6,7,5,1,3,5,7,5,1,7,8,2,9,6,6,
 6,4,5,4,7,7,9,1,7,4,5,0,1,1,2,9,9,6,1,4,8,9,0,3,0,4,6,3,9,9,4,7,1,3,2,9,6,2,
 1,0,7,3,4,0,4,3,7,5,1,8,9,5,7,3,5,9,6,1,4,5,8,9,0,1,9,3,8,9,7,1,3,1,1,1,7,9,
 0,4,2,9,7,8,2,8,5,6,4,7,5,0,3,2,0,3,1,9,8,6,9,1,5,1,4,0,2,8,7,0,8,0,8,5,9,9,
 0,4,8,0,1,0,9,4,1,2,1,4,7,2,2,1,3,1,7,9,4,7,6,4,7,7,7,2,6,2,2,4,1,4,2,5,4,8,
 5,4,5,4,0,3,3,2,1,5,7,1,8,5,3,0,6,1,4,2,2,8,8,1,3,7,5,8,5,0,4,3,0,6,3,3,2,1,
 7,5,1,8,2,9,7,9,8,6,6,2,2,3,7,1,7,2,1,5,9,1,6,0,7,7,1,6,6,9,2,5,4,7,4,8,7,3,
 8,9,8,6,6,5,4,9,4,9,4,5,0,1,1,4,6,5,4,0,6,2,8,4,3,3,6,6,3,9,3,7,9,0,0,3,9,7,
 6,9,2,6,5,6,7,2,1,4,6,3,8,5,3,0,6,7,3,6,0,9,6,5,7,1,2,0,9,1,8,0,7,6,3,8,3,2,
 7,1,6,6,4,1,6,2,7,4,8,8,8,8,0,0,7,8,6,9,2,5,6,0,2,9,0,2,2,8,4,7,2,1,0,4,0,3,
 1,7,2,1,1,8,6,0,8,2,0,4,1,9,0,0,0,4,2,2,9,6,6,1,7,1,1,9,6,3,7,7,9,2,1,3,3,7,
 5,7,5,1,1,4,9,5,9,5,0,1,5,6,6,0,4,9,6,3,1,8,6,2,9,4,7,2,6,5,4,7,3,6,4,2,5,2,
 3,0,8,1,7,7,0,3,6,7,5,1,5,9,0,6,7,3,5,0,2,3,5,0,7,2,8,3,5,4,0,5,6,7,0,4,0,3,
 8,6,7,4,3,5,1,3,6,2,2,2,2,4,7,7,1,5,8,9,1,5,0,4,9,5,3,0,9,8,4,4,4,8,9,3,3,3,
 0,9,6,3,4,0,8,7,8,0,7,6,9,3,2,5,9,9,3,9,7,8,0,5,4,1,9,3,4,1,4,4,7,3,7,7,4,4,
 1,8,4,2,6,3,1,2,9,8,6,0,8,0,9,9,8,8,8,6,8,7,4,1,3,2,6,0,4,7,2,1,5,6,9,5,1,6,
 2,3,9,6,5,8,6,4,5,7,3,0,2,1,6,3,1,5,9,8,1,9,3,1,9,5,1,6,7,3,5,3,8,1,2,9,7,4,
 1,6,7,7,2,9,4,7,8,6,7,2,4,2,2,9,2,4,6,5,4,3,6,6,8,0,0,9,8,0,6,7,6,9,2,8,2,3,
 8,2,8,0,6,8,9,9,6,4,0,0,4,8,2,4,3,5,4,0,3,7,0,1,4,1,6,3,1,4,9,6,5,8,9,7,9,4,
 0,9,2,4,3,2,3,7,8,9,6,9,0,7,0,6,9,7,7,9,4,2,2,3,6,2,5,0,8,2,2,1,6,8,8,9,5,7);

xl10: array [1..5166] of byte =
(1,0,20,1,0,255,19,2,3,0,2,5,8,5,0,9,2,9,9,4,0,4,5,6,8,4,0,1,7,9,9,1,4,5,4,6,
 8,4,3,6,4,2,0,7,6,0,1,1,0,1,4,8,8,6,2,8,7,7,2,9,7,6,0,3,3,3,2,7,9,0,0,9,6,7,
 5,7,2,6,0,9,6,7,7,3,5,2,4,8,0,2,3,5,9,9,7,2,0,5,0,8,9,5,9,8,2,9,8,3,4,1,9,6,
 7,7,8,4,0,4,2,2,8,6,2,4,8,6,3,3,4,0,9,5,2,5,4,6,5,0,8,2,8,0,6,7,5,6,6,6,6,2,
 8,7,3,6,9,0,9,8,7,8,1,6,8,9,4,8,2,9,0,7,2,0,8,3,2,5,5,5,4,6,8,0,8,4,3,7,9,9,
 8,9,4,8,2,6,2,3,3,1,9,8,5,2,8,3,9,3,5,0,5,3,0,8,9,6,5,3,7,7,7,3,2,6,2,8,8,4,
 6,1,6,3,3,6,6,2,2,2,2,8,7,6,9,8,2,1,9,8,8,6,7,4,6,5,4,3,6,6,7,4,7,4,4,0,4,2,
 4,3,2,7,4,3,6,5,1,5,5,0,4,8,9,3,4,3,1,4,9,3,9,3,9,1,4,7,9,6,1,9,4,0,4,4,0,0,
 2,2,2,1,0,5,1,0,1,7,1,4,1,7,4,8,0,0,3,6,8,8,0,8,4,0,1,2,6,4,7,0,8,0,6,8,5,5,
 6,7,7,4,3,2,1,6,2,2,8,3,5,5,2,2,0,1,1,4,8,0,4,6,6,3,7,1,5,6,5,9,1,2,1,3,7,3,
 4,5,0,7,4,7,8,5,6,9,4,7,6,8,3,4,6,3,6,1,6,7,9,2,1,0,1,8,0,6,4,4,5,0,7,0,6,4,
 8,0,0,0,2,7,7,5,0,2,6,8,4,9,1,6,7,4,6,5,5,0,5,8,6,8,5,6,9,3,5,6,7,3,4,2,0,6,
 7,0,5,8,1,1,3,6,4,2,9,2,2,4,5,5,4,4,0,5,7,5,8,9,2,5,7,2,4,2,0,8,2,4,1,3,1,4,
 6,9,5,6,8,9,0,1,6,7,5,8,9,4,0,2,5,6,7,7,6,3,1,1,3,5,6,9,1,9,2,9,2,0,3,3,3,7,
 6,5,8,7,1,4,1,6,6,0,2,3,0,1,0,5,7,0,3,0,8,9,6,3,4,5,7,2,0,7,5,4,4,0,3,7,0,8,
 4,7,4,6,9,9,4,0,1,6,8,2,6,9,2,8,2,8,0,8,4,8,1,1,8,4,2,8,9,3,1,4,8,4,8,5,2,4,
 9,4,8,6,4,4,8,7,1,9,2,7,8,0,9,6,7,6,2,7,1,2,7,5,7,7,5,3,9,7,0,2,7,6,6,8,6,0,
 5,9,5,2,4,9,6,7,1,6,6,7,4,1,8,3,4,8,5,7,0,4,4,2,2,5,0,7,1,9,7,9,6,5,0,0,4,7,
 1,4,9,5,1,0,5,0,4,9,2,2,1,4,7,7,6,5,6,7,6,3,6,9,3,8,6,6,2,9,7,6,9,7,9,5,2,2,
 1,1,0,7,1,8,2,6,4,5,4,9,7,3,4,7,7,2,6,6,2,4,2,5,7,0,9,4,2,9,3,2,2,5,8,2,7,9,
 8,5,0,2,5,8,5,5,0,9,7,8,5,2,6,5,3,8,3,2,0,7,6,0,6,7,2,6,3,1,7,1,6,4,3,0,9,5,
 0,5,9,9,5,0,8,7,8,0,7,5,2,3,7,1,0,3,3,3,1,0,1,1,9,7,8,5,7,5,4,7,3,3,1,5,4,1,
 4,2,1,8,0,8,4,2,7,5,4,3,8,6,3,5,9,1,7,7,8,1,1,7,0,5,4,3,0,9,8,2,7,4,8,2,3,8,
 5,0,4,5,6,4,8,0,1,9,0,9,5,6,1,0,2,9,9,2,9,1,8,2,4,3,1,8,2,3,7,5,2,5,3,5,7,7,
 0,9,7,5,0,5,3,9,5,6,5,1,8,7,6,9,7,5,1,0,3,7,4,9,7,0,8,8,8,6,9,2,1,8,0,2,0,5,
 1,8,9,3,3,9,5,0,7,2,3,8,5,3,9,2,0,5,1,4,4,6,3,4,1,9,7,2,6,5,2,8,7,2,8,6,9,6,
 5,1,1,0,8,6,2,5,7,1,4,9,2,1,9,8,8,4,9,9,7,8,7,4,8,8,7,3,7,7,1,3,4,5,6,8,6,2,
 0,9,1,6,7,0,5,8,4,9,8,0,7,8,2,8,0,5,9,7,5,1,1,9,3,8,5,4,4,4,5,0,0,9,9,7,8,1,
 3,1,1,4,6,9,1,5,9,3,4,6,6,6,2,4,1,0,7,1,8,4,6,6,9,2,3,1,0,1,0,7,5,9,8,4,3,8,
 3,1,9,1,9,1,2,9,2,2,3,0,7,9,2,5,0,3,7,4,7,2,9,8,6,5,0,9,2,9,0,0,9,8,8,0,3,9,
 1,9,4,1,7,0,2,6,5,4,4,1,6,8,1,6,3,3,5,7,2,7,5,5,5,7,0,3,1,5,1,5,9,6,1,1,3,5,
 6,4,8,4,6,5,4,6,1,9,0,8,9,7,0,4,2,8,1,9,7,6,3,3,6,5,8,3,6,9,8,3,7,1,6,3,2,8,
 9,8,2,1,7,4,4,0,7,3,6,6,0,0,9,1,6,2,1,7,7,8,5,0,5,4,1,7,7,9,2,7,6,3,6,7,7,3,
 1,1,4,5,0,4,1,7,8,2,1,3,7,6,6,0,1,1,1,0,1,0,7,3,1,0,4,2,3,9,7,8,3,2,5,2,1,8,
 9,4,8,9,8,8,1,7,5,9,7,9,2,1,7,9,8,6,6,6,3,9,4,3,1,9,5,2,3,9,3,6,8,5,5,9,1,6,
 4,4,7,1,1,8,2,4,6,7,5,3,2,4,5,6,3,0,9,1,2,5,2,8,7,7,8,3,3,0,9,6,3,6,0,4,2,6,
 2,9,8,2,1,5,3,0,4,0,8,7,4,5,6,0,9,2,7,7,6,0,7,2,6,6,4,1,3,5,4,7,8,7,5,7,6,6,
 1,6,2,6,2,9,2,6,5,6,8,2,9,8,7,0,4,9,5,7,9,5,4,9,1,3,9,5,4,9,1,8,0,4,9,2,0,9,
 0,6,9,4,3,8,5,8,0,7,9,0,0,3,2,7,6,3,0,1,7,9,4,1,5,0,3,1,1,7,8,6,6,8,6,2,0,9,
 2,4,0,8,5,3,7,9,4,9,8,6,1,2,6,4,9,3,3,4,7,9,3,5,4,8,7,1,7,3,7,4,5,1,6,7,5,8,
 0,9,5,3,7,0,8,8,2,8,1,0,6,7,4,5,2,4,4,0,1,0,5,8,9,2,4,4,4,9,7,6,4,7,9,6,8,6,
 0,7,5,1,2,0,2,7,5,7,2,4,1,8,1,8,7,4,9,8,9,3,9,5,9,7,1,6,4,3,1,0,5,5,1,8,8,4,
 8,1,9,5,2,8,8,3,3,0,7,4,6,6,9,9,3,1,7,8,1,4,6,3,4,9,3,0,0,0,0,3,2,1,2,0,0,3,
 2,7,7,6,5,6,5,4,1,3,0,4,7,2,6,2,1,8,8,3,9,7,0,5,9,6,7,9,4,4,5,7,9,4,3,4,6,8,
 3,4,3,2,1,8,3,9,5,3,0,4,4,1,4,8,4,4,8,0,3,7,0,1,3,0,5,7,5,3,6,7,4,2,6,2,1,5,
 3,6,7,5,5,7,9,8,1,4,7,7,0,4,5,8,0,3,1,4,1,3,6,3,7,7,9,3,2,3,6,2,9,1,5,6,0,1,
 2,8,1,8,5,3,3,6,4,9,8,4,6,6,9,4,2,2,6,1,4,6,5,2,0,6,4,5,9,9,4,2,0,7,2,9,1,7,
 1,1,9,3,7,0,6,0,2,4,4,4,9,2,9,3,5,8,0,3,7,0,0,7,7,1,8,9,8,1,0,9,7,3,6,2,5,3,
 3,2,2,4,5,4,8,3,6,6,9,8,8,5,0,5,5,2,8,2,8,5,9,6,6,1,9,2,8,0,5,0,9,8,4,4,7,1,
 7,5,1,9,8,5,0,3,6,6,6,6,8,0,8,7,4,9,7,0,4,9,6,9,8,2,2,7,3,2,2,0,2,4,4,8,2,3,
 3,4,3,0,9,7,1,6,9,1,1,1,1,3,6,8,1,3,5,8,8,4,1,8,6,9,6,5,4,9,3,2,3,7,1,4,9,9,
 6,9,4,1,9,7,9,6,8,7,8,0,3,0,0,8,8,5,0,4,0,8,9,7,9,6,1,8,5,9,8,7,5,6,5,7,9,8,
 9,4,8,3,6,4,4,5,2,1,2,0,4,3,6,9,8,2,1,6,4,1,5,2,9,2,9,8,7,8,1,1,7,4,2,9,7,3,
 3,3,2,5,8,8,6,0,7,9,1,5,9,1,2,5,1,0,9,6,7,1,8,7,5,1,0,9,2,9,2,4,8,4,7,5,0,2,
 3,9,3,0,5,7,2,6,6,5,4,4,6,2,7,6,2,0,0,9,2,3,0,6,8,7,9,1,5,1,8,1,3,5,8,0,3,4,
 7,7,7,0,1,2,9,5,5,9,3,6,4,6,2,9,8,4,1,2,3,6,6,4,9,7,0,2,3,3,5,5,1,7,4,5,8,6,
 1,9,5,5,6,4,7,7,2,4,6,1,8,5,7,7,1,7,3,6,9,3,6,8,4,0,4,6,7,6,5,7,7,0,4,7,8,7,
 4,3,1,9,7,8,0,5,7,3,8,5,3,2,7,1,8,1,0,9,3,3,8,8,3,4,9,6,3,3,8,8,1,3,0,6,9,9,
 4,5,5,6,9,3,9,9,3,4,6,1,0,1,0,9,0,7,4,5,6,1,6,0,3,3,3,1,2,2,4,7,9,4,9,3,6,0,
 4,5,5,3,6,1,8,4,9,1,2,3,3,3,3,0,6,3,7,0,4,7,5,1,7,2,4,8,7,1,2,7,6,3,7,9,1,4,
 0,9,2,4,3,9,8,3,3,1,8,1,0,1,6,4,7,3,7,8,2,3,3,7,9,6,9,2,2,6,5,6,3,7,6,8,2,0,
 7,1,7,0,6,9,3,5,8,4,6,3,9,4,5,3,1,6,1,6,9,4,9,4,1,1,7,0,1,8,4,1,9,3,8,1,1,9,
 4,0,5,4,1,6,4,4,9,4,6,6,1,1,1,2,7,4,7,1,2,8,1,9,7,0,5,8,1,7,7,8,3,2,9,3,8,4,
 1,7,4,2,2,3,1,4,0,9,9,3,0,0,2,2,9,1,1,5,0,2,3,6,2,1,9,2,1,8,6,7,2,3,3,3,7,2,
 6,8,3,8,5,6,8,8,2,7,3,5,3,3,3,7,1,9,2,5,1,0,3,4,1,2,9,3,0,7,0,5,6,3,2,5,4,4,
 4,2,6,6,1,1,4,2,9,7,6,5,3,8,8,3,0,1,8,2,2,3,8,4,0,9,1,0,2,6,1,9,8,5,8,2,8,8,
 8,4,3,3,5,8,7,4,5,5,9,6,0,4,5,3,0,0,4,5,4,8,3,7,0,7,8,9,0,5,2,5,7,8,4,7,3,1,
 6,6,2,8,3,7,0,1,9,5,3,3,9,2,2,3,1,0,4,7,5,2,7,5,6,4,9,9,8,1,1,9,2,2,8,7,4,2,
 7,8,9,7,1,3,7,1,5,7,1,3,2,2,8,3,1,9,6,4,1,0,0,3,4,2,2,1,2,4,2,1,0,0,8,2,1,8,
 0,6,7,9,5,2,5,2,7,6,6,8,9,8,5,8,1,8,0,9,5,6,1,1,9,2,0,8,3,9,1,7,6,0,7,2,1,0,
 8,0,9,1,9,9,2,3,4,6,1,5,1,6,9,5,2,5,9,9,0,9,9,4,7,3,7,8,2,7,8,0,6,4,8,1,2,8,
 0,5,8,7,9,2,7,3,1,9,9,3,8,9,3,4,5,3,4,1,5,3,2,0,1,8,5,9,6,9,7,1,1,0,2,1,4,0,
 7,5,4,2,2,8,2,7,9,6,2,9,8,2,3,7,0,6,8,9,4,1,7,6,4,7,4,0,6,4,2,2,2,5,7,5,7,2,
 1,2,4,5,5,3,9,2,5,2,6,1,7,9,3,7,3,6,5,2,4,3,4,4,4,0,5,6,0,5,9,5,3,3,6,5,9,1,
 5,3,9,1,6,0,3,1,2,5,2,4,4,8,0,1,4,9,3,1,3,2,3,4,5,7,2,4,5,3,8,7,9,5,2,4,3,8,
 9,0,3,6,8,3,9,2,3,6,4,5,0,5,0,7,8,8,1,7,3,1,3,5,9,7,1,1,2,3,8,1,4,5,3,2,3,7,
 0,1,5,0,8,4,1,3,4,9,1,1,2,2,3,2,4,3,9,0,9,2,7,6,8,1,7,2,4,7,4,9,6,0,7,9,5,5,
 7,9,9,1,5,1,3,6,3,9,8,2,8,8,1,0,5,8,2,8,5,7,4,0,5,3,8,0,0,0,6,5,3,3,7,1,6,5,
 5,5,5,3,0,1,4,1,9,6,3,3,2,2,4,1,9,1,8,0,8,7,6,2,1,0,1,8,2,0,4,9,1,9,4,9,2,6,
 5,1,4,8,3,8,9,2,6,9,2,2,9,3,7,0,7,8,9,8,6,3,5,2,7,0,6,3,8,5,0,2,7,0,2,2,6,9,
 7,5,5,1,2,4,9,9,4,3,0,0,4,3,8,1,8,2,7,8,8,4,7,3,1,9,9,2,1,9,6,5,8,1,4,3,1,2,
 1,9,7,6,0,7,1,1,1,0,8,7,6,3,4,1,2,2,4,4,6,6,4,0,7,3,7,7,9,4,5,5,5,8,8,5,8,3,
 9,1,9,4,4,0,8,0,9,3,1,1,0,2,4,2,8,3,0,8,5,0,8,4,7,4,6,3,1,9,0,6,9,3,2,1,9,8,
 1,4,6,8,6,4,3,0,8,3,5,5,5,3,4,7,6,5,1,1,1,4,6,5,0,7,6,9,0,9,7,3,3,3,3,9,5,8,
 3,2,5,7,5,0,6,8,2,7,7,5,8,2,8,3,1,0,5,1,8,0,3,2,9,9,4,8,1,3,0,8,4,8,3,9,9,6,
 8,1,7,2,1,1,3,6,6,7,9,8,8,0,1,9,2,0,3,7,7,7,0,6,8,6,8,5,4,7,9,5,0,6,2,0,3,5,
 8,4,1,7,2,1,9,9,8,4,3,7,4,3,9,4,1,0,6,3,6,2,3,4,3,2,3,1,4,8,1,1,7,4,5,5,2,4,
 7,5,6,4,0,0,8,9,1,9,2,1,6,5,4,9,1,2,6,9,0,8,2,7,8,1,5,0,6,0,4,8,1,7,1,1,9,9,
 0,7,0,1,8,8,6,2,9,2,0,9,8,2,2,8,6,2,7,1,6,7,3,0,4,7,7,1,9,9,0,1,5,7,7,0,2,8,
 5,3,5,7,9,1,0,5,0,6,2,8,4,6,7,3,3,7,2,2,3,2,4,0,0,0,1,8,7,9,5,6,1,5,0,2,2,8,
 9,0,8,4,3,9,6,7,4,1,4,6,8,5,6,5,6,4,8,3,1,7,6,8,8,1,3,8,9,2,0,6,8,4,0,9,5,7,
 9,6,9,1,8,0,3,3,5,4,3,3,9,1,3,6,4,6,7,8,6,5,7,4,5,9,0,5,4,9,4,8,8,0,2,9,8,5,
 4,3,8,1,9,8,8,0,7,3,5,9,5,5,4,2,7,0,4,1,5,4,0,8,7,3,4,0,1,4,5,3,1,7,3,9,2,5,
 4,5,3,6,7,3,2,8,1,9,3,9,9,7,6,4,5,6,0,2,5,5,9,8,4,3,2,4,3,3,0,5,6,1,5,8,7,3,
 5,0,0,2,9,3,2,8,0,2,5,7,2,3,0,6,4,5,3,2,7,0,3,4,8,1,1,8,1,0,8,4,7,2,3,1,1,6,
 4,4,6,8,0,4,7,5,1,9,1,9,4,2,6,4,1,4,4,3,7,5,5,4,8,6,5,7,6,9,3,9,2,9,2,9,6,8,
 3,4,2,8,6,6,7,5,8,5,1,0,6,7,3,1,9,2,4,1,9,1,8,2,2,8,5,2,2,6,8,9,9,1,3,8,9,3,
 1,1,9,2,1,6,9,4,4,2,4,2,3,0,3,8,5,1,7,0,6,5,2,9,3,3,7,0,4,5,1,2,1,4,9,8,3,8,
 3,5,9,7,9,4,0,2,1,8,2,7,1,6,2,9,1,3,3,6,4,0,2,5,3,1,5,4,9,5,9,6,9,1,3,1,1,7,
 1,4,6,6,3,5,3,4,4,2,2,2,6,4,5,6,8,5,4,4,6,8,3,2,2,6,2,8,9,2,5,3,1,9,8,5,2,7,
 3,9,9,4,6,6,6,6,9,5,7,1,8,6,0,1,5,7,9,2,9,8,3,8,5,0,4,4,0,1,5,3,7,4,6,1,5,2,
 8,2,7,5,1,2,8,8,5,7,5,4,5,6,1,7,5,4,1,9,6,9,9,7,6,7,8,9,0,8,9,2,1,1,7,2,4,9,
 0,2,3,7,6,3,4,7,8,7,8,9,4,2,4,0,2,1,8,3,3,7,6,7,8,1,3,9,7,3,5,4,6,5,8,3,4,5,
 5,5,6,2,9,5,0,7,9,3,0,9,0,4,6,6,8,0,5,1,5,9,7,6,4,0,9,5,0,5,2,4,8,1,8,9,4,7,
 5,8,7,3,0,9,9,4,7,6,6,1,2,9,8,6,3,2,0,6,7,7,3,5,6,5,3,1,1,1,1,7,8,6,0,6,1,5,
 3,7,4,8,1,0,1,0,8,1,5,8,6,4,2,7,7,2,1,1,1,1,3,5,0,0,1,7,6,6,6,1,0,9,1,1,4,9,
 6,9,6,4,9,4,1,8,3,0,7,6,9,4,6,3,0,1,3,8,8,4,1,6,0,2,6,5,0,3,1,7,8,4,8,1,1,9,
 2,9,7,2,3,8,0,2,0,6,8,0,8,6,8,9,6,2,3,6,5,4,3,9,6,5,9,1,5,1,2,0,6,7,6,0,7,9,
 0,5,0,0,8,3,1,9,5,2,3,0,2,4,3,4,7,8,1,7,8,8,3,0,3,8,7,7,8,4,7,1,0,2,8,2,6,3,
 0,5,5,9,1,2,4,3,0,6,2,4,3,9,4,3,1,5,6,9,1,1,9,4,6,1,7,2,3,3,2,1,5,1,3,0,4,7,
 8,3,2,0,8,9,4,2,5,8,5,1,3,0,5,2,6,4,2,5,1,8,2,1,1,4,7,9,8,4,8,2,5,8,4,2,4,7,
 3,0,9,6,7,0,7,9,5,0,1,6,4,4,3,9,5,5,2,0,0,6,3,1,3,7,1,1,4,2,4,8,5,0,0,2,4,9,
 7,6,5,2,9,0,1,5,1,3,1,0,3,8,0,2,1,6,4,1,6,8,4,3,4,6,8,0,3,6,6,2,3,9,6,5,0,4,
 0,0,5,7,8,5,2,0,3,1,7,2,7,4,6,4,6,3,4,1,4,8,8,8,9,4,1,2,7,7,0,3,8,8,3,1,8,7,
 6,0,6,6,6,9,0,4,1,7,2,9,6,5,9,7,1,3,5,9,9,0,8,1,7,7,3,6,9,0,0,6,3,4,7,8,9,4,
 5,9,5,2,8,8,9,7,6,2,2,1,9,9,7,8,8,0,7,9,8,2,7,1,3,9,8,3,3,1,8,1,5,3,9,5,5,4,
 1,5,7,2,9,7,8,9,4,7,0,1,6,7,2,8,7,5,0,2,5,3,2,3,5,9,0,8,2,7,8,1,3,1,6,3,5,7,
 0,6,6,6,3,4,8,9,5,6,7,6,3,6,6,3,4,4,0,0,1,1,3,0,0,9,8,0,0,9,3,8,7,8,3,5,9,2,
 1,3,0,0,2,6,2,9,5,3,0,7,1,2,3,6,7,4,3,0,2,2,3,2,3,9,9,7,0,7,3,3,7,9,8,6,6,8,
 4,1,5,1,7,1,8,2,4,6,8,8,8,1,4,5,6,5,7,6,4,7,7,2,1,1,7,5,1,1,6,3,6,7,7,9,7,3,
 2,7,5,0,5,1,8,3,0,7,4,9,0,2,6,6,5,3,6,2,4,0,6,0,6,7,7,8,9,6,1,2,9,3,1,3,0,6,
 8,5,4,9,7,5,8,7,4,3,3,0,5,4,8,5,3,6,9,3,7,6,6,5,6,3,0,3,1,6,7,1,1,1,1,7,8,7,
 1,5,6,4,5,7,1,1,5,6,4,0,2,4,9,9,7,8,3,0,5,3,4,9,2,2,8,9,7,8,0,2,6,8,4,6,4,6,
 0,8,1,9,2,1,3,2,8,4,0,3,1,4,8,0,4,8,2,7,4,9,8,9,3,6,9,7,0,3,7,3,2,8,0,9,3,3,
 1,4,5,4,0,1,0,1,9,2,7,6,4,8,3,4,0,6,0,5,9,3,7,1,5,1,4,1,8,5,7,9,8,9,3,3,5,6,
 4,1,5,9,4,8,3,2,3,8,3,5,3,2,5,3,2,8,7,5,2,0,1,0,0,8,5,5,5,5,5,0,1,5,9,2,8,0,
 8,9,1,8,3,3,2,6,8,0,8,9,9,3,6,8,3,2,7,5,1,6,1,3,7,4,3,8,9,8,1,9,7,3,6,1,2,5,
 7,0,6,6,7,9,6,2,7,1,6,1,7,8,9,8,2,0,1,6,8,0,8,5,7,6,1,1,0,3,7,0,6,6,0,8,2,1,
 5,7,5,3,2,7,0,0,0,1,3,4,4,1,4,5,7,9,9,8,8,3,9,0,0,7,7,0,9,0,5,7,7,9,0,9,4,7,
 6,9,3,3,3,8,7,1,8,5,9,3,0,5,5,7,4,5,1,6,3,7,2,1,6,3,9,2,2,3,0,2,7,8,8,5,4,6,
 8,8,4,9,9,2,1,9,1,7,4,7,7,5,3,6,2,0,1,3,5,7,4,0,0,0,7,3,8,0,8,0,2,9,0,0,3,4,
 6,4,8,0,1,2,8,9,5,0,5,0,6,3,8,5,6,8,7,1,4,9,6,7,9,7,0,2,7,8,2,4,2,1,3,3,9,8,
 6,9,0,3,6,6,6,4,7,6,9,2,4,2,1,3,8,7,2,2,1,0,4,7,4,7,9,3,2,9,4,2,6,7,4,6,9,4,
 0,6,3,1,1,2,6,3,8,4,9,1,0,1,4,3,6,1,8,0,7,2,8,7,7,6,6,0,8,0,4,1,0,5,1,2,5,1,
 2,2,0,3,8,5,7,7,9,4,2,9,0,6,3,7,7,3,0,0,6,9,1,2,4,1,7,0,7,4,2,8,8,2,0,6,6,9,
 7,9,8,6,1,0,7,7,7,7,4,3,0,2,4,5,0,4,3,0,7,8,9,5,7,8,0,0,7,8,7,3,9,4,0,2,7,3);

pow2 : array[0..31] of longword =
(1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536,
131072,262144,524288,1048576,2097152,4194304,8388608,16777216,33554432,
67108864,134217728,268435456,536870912,1073741824,2147483648);

base: qword = 4294967296;

VAR

ub1,ub2,ub3: vbuffer;
ac: array [0..20] of byte;
mpc,mpq: qword;
b1,b2,b3,b4,b5,b6,b7: bufnum;
rc,c2,c3,c4,c5,c6,c7: extnum;
s,s1: ansistring;
r1,d,t,b,z: byte;
r: shortint;
nc,i,j,x,y,k1,dx,k2,dy,ncqur,ndqur,m1,m2,ncvr,ip: word;
m,p,ndvr,ncfr,ndfr,ncrq,ncmq,ncpq,n,c,hq,n1,n2,n3,zp,np,mpw: word;
md1,md2,df: longword;
h: qword;
rf,uf,ef,vf: longword;
qdr,sss,powsign: byte;
subflag: boolean;
sg1,sg2,sg3: byte;
nd: smallint;
mpg,cce: longword;
ia3,da3: word;
ch: char;
kf,lv: word;
q,k: longword;
ncprd,niprd,ndprd: word;
ix,jv,kx,iq: word;
rb,bk: byte;
tb: qword;
bf,bg,bf1,bf2: nbuffer;
ml,pl: longword;
zl: byte;
kv: longint;
ux: byte;
sgdr1,sgdr2,sgqur: byte;
ns,ndv,nrr: word;
qr,kr: longword;
{for special functions}
kz,erfprec: word;
rz,uz,vz: longword;
a2,a3,a4,a5,a6,a7,c8,rd,ez,v2,v3,v4,v5,v6,v7,v8,vp: extnum;
xz: extended;
uk: word;

{* internal procedures *}

procedure recip;    {* calculates in rc the reciprocal of uf *}
begin
kf:=1; rf:=10;
while uf>rf do begin
rf:=rf*10; inc(kf); rc.c[kf]:=0
end;
while (kf<usrdigits) and (rf>0) do begin
inc(kf);
vf:=rf div uf;
rf:=(rf-vf*uf)*10;
rc.c[kf]:=vf
end;
rc.n:=kf; rc.d:=kf-1
end;

procedure quot;     {* calculates in rc the quotient ef/uf *}
begin
kf:=1; rf:=ef*10;
while uf>rf do begin
rf:=rf*10; inc(kf); rc.c[kf]:=0
end;
while (kf<usrdigits) and (rf>0) do begin
inc(kf);
vf:=rf div uf;
rf:=(rf-vf*uf)*10;
rc.c[kf]:=vf
end;
rc.n:=kf; rc.d:=kf-1
end;

{* a conversion of the integer part to buffers of longwords in base 2^32
is useful to calculate the logical functions and, or, xor and not *}

{* from buffer to extnum *}

procedure buftoex(b: nbuffer; var ex: extnum);
begin
if (b[0]=1) and (b[1]=0) then begin
move(ar0,ex,8);
exit
end;
bf[0]:=b[0];
for ix:=1 to b[0] do bf[ix]:=b[b[0]+1-ix];
kx:=0;
jv:=bf[0];
while jv>0 do begin
rb:=0;
for ix:=1 to jv do begin
tb:=rb*base+bf[ix];
bf[ix]:=tb div 10;
rb:=tb mod 10
end;
if bf[1]=0 then begin
dec(jv);
for ix:=1 to jv do bf[ix]:=bf[ix+1];
dec(bf[0])
end;
inc(kx);
b6[kx]:=rb
end;
ex.s:=1;
ex.n:=kx;
ex.i:=kx;
ex.d:=0;
for ix:=1 to kx do ex.c[ix]:=b6[kx-ix+1]
end;

{* from extnum to buffer *}

procedure extobuf(ex: extnum; var b: nbuffer);
begin
b[1]:=0;
jv:=1;
for iq:=1 to ex.i do begin
kx:=0;
for ix:=jv downto 1 do begin
tb:=b[ix];
tb:=tb*10+kx;
b[ix]:=tb and 4294967295;
kx:=tb shr 32
end;
if kx>0 then begin
inc(jv);
for ix:=jv downto 1 do b[ix+1]:=b[ix];
b[1]:=kx
end;
bk:=ex.c[iq];
rb:=0;
for ix:=jv downto 1 do begin
tb:=b[ix]+bk+rb;
bk:=0;
if tb>=base then rb:=1 else rb:=0;
b[ix]:=tb and 4294967295
end;
if rb>0 then begin
for ix:=jv downto 1 do b[ix+1]:=b[ix];
b[1]:=rb;
inc(jv)
end
end;
for ix:=1 to jv do bg[ix]:=b[jv+1-ix];
for ix:=1 to jv do b[ix]:=bg[ix];
while (b[jv]=0) and (jv>1) do dec(jv);
b[0]:=jv
end;

{addition of a byte 0..9 to a positive number}

procedure xaddb (var e: extnum; b: byte);
begin
i:=e.i;
inc(e.c[i],b);
if e.c[i]>9 then begin
while (i>1) and (e.c[i]>9) do begin
dec(e.c[i],10);
dec(i);
inc(e.c[i])
end;
if e.c[1]>9 then begin
dec(e.c[1],10);
for i:=e.i downto 1 do e.c[i+1]:=e.c[i];
e.c[1]:=1;
inc(e.i);
inc(e.n)
end
end
end;

{multiplication by a byte 0..9}

procedure xmulb(e1: extnum; b: byte; var e2:extnum);
begin
z:=0;
for i:=e1.n downto 1 do begin
d:=e1.c[i]*b+z;
z:=d div 10;
e2.c[i]:=d mod 10
end;
e2.n:=e1.n;
e2.i:=e1.i;
e2.d:=e1.d;
e2.s:=e1.s;
if z>0 then begin
for i:=e1.n downto 1 do e2.c[i+1]:=e2.c[i];
e2.c[1]:=z;
inc(e2.n);
inc(e2.i)
end;
while (e2.c[e2.n]=0) and (e2.d>0) do begin
dec(e2.d);
dec(e2.n)
end;
if e2.i>usrdigits then xstatus:=7
end;

{multiplication by a byte 0..9}

procedure xmulbs(var e1: extnum; b: byte);
begin
z:=0;
for i:=e1.n downto 1 do begin
d:=e1.c[i]*b+z;
z:=d div 10;
e1.c[i]:=d mod 10
end;
if z>0 then begin
for i:=e1.n downto 1 do e1.c[i+1]:=e1.c[i];
e1.c[1]:=z;
inc(e1.n);
inc(e1.i)
end;
while (e1.c[e1.n]=0) and (e1.d>0) do begin
dec(e1.d);
dec(e1.n)
end;
if e1.i>usrdigits then xstatus:=7
end;

{division by a byte 1..9}

procedure xdivb(e1:extnum; b:byte; var e2:extnum);
begin
i:=1;
j:=0;
d:=e1.c[1];
if (e1.i>1) and (d<b) then begin
d:=d*10+e1.c[2];
inc(i)
end;
inc(j);
e2.c[j]:=d div b;
rb:=d mod b;
while (i<e1.i) do begin
inc(i);
inc(j);
d:=rb*10+e1.c[i];
e2.c[j]:=d div b;
rb:=d mod b
end;
e2.i:=j;
if e1.d=0 then begin
if rb=0 then begin
e2.n:=j;
e2.d:=0;
e2.s:=e1.s;
exit
end
else begin
while (rb>0) and (j<usrdigits) do begin
inc(j);
d:=rb*10;
e2.c[j]:=d div b;
rb:=d mod b
end;
e2.n:=j;
e2.d:=j-e2.i;
e2.s:=e1.s;
exit
end
end
else begin
uk:=j+e1.d;
while j<uk do begin
inc(i);
inc(j);
d:=rb*10+e1.c[i];
e2.c[j]:=d div b;
rb:=d mod b
end;
while (rb>0) and (j<usrdigits) do begin
inc(j);
d:=rb*10;
e2.c[j]:=d div b;
rb:=d mod b
end;
e2.n:=j;
e2.d:=j-e2.i;
e2.s:=e1.s
end
end;

{division by a byte 1..9}

procedure xdivbs(var e1:extnum; b:byte);
begin
i:=1;
j:=0;
d:=e1.c[1];
if (e1.i>1) and (d<b) then begin
d:=d*10+e1.c[2];
inc(i)
end;
inc(j);
e1.c[j]:=d div b;
rb:=d mod b;
while (i<e1.i) do begin
inc(i);
d:=rb*10+e1.c[i];
inc(j);
e1.c[j]:=d div b;
rb:=d mod b
end;
e1.i:=j;
if e1.d=0 then begin
if rb=0 then begin
e1.n:=j;
e1.d:=0;
exit
end
else begin
while (rb>0) and (j<usrdigits) do begin
inc(j);
d:=rb*10;
e1.c[j]:=d div b;
rb:=d mod b
end;
e1.n:=j;
e1.d:=j-e1.i;
exit
end
end
else begin
uk:=j+e1.d;
while j<uk do begin
inc(i);
d:=rb*10+e1.c[i];
inc(j);
e1.c[j]:=d div b;
rb:=d mod b
end;
while (rb>0) and (j<usrdigits) do begin
inc(j);
d:=rb*10;
e1.c[j]:=d div b;
rb:=d mod b
end;
e1.n:=j;
e1.d:=j-e1.i
end
end;

{* interface procedures and functions *}

procedure xtox(e1: extnum; var e2: extnum);
begin
move(e1,e2,e1.n+7)
end;

procedure xelz(var e:extnum);
begin
if (e.c[e.n]=0) and (e.d>0) then begin
while (e.c[e.n]=0) and (e.n>e.i) do dec(e.n);
e.d:=e.n-e.i
end;
j:=1;
while (e.c[j]=0) and (j<e.i) do inc(j);
if j>1 then begin
dec(j);
dec(e.n,j); dec(e.i,j);
for i:=1 to e.n do e.c[i]:=e.c[i+j]
end
end;

procedure xsetn(n: word);
begin
if n<10 then usrdigits:=10
else if n>maxdigits then usrdigits:=maxdigits
else usrdigits:=n
end;

procedure xdisplay(e: extnum);
begin
if (e.n>1) or (e.c[1]>0) then
if e.s=1 then write('+') else write('-');
for j:=1 to e.i do write(e.c[j]);
if e.d>0 then begin
write('.');
for j:=e.i+1 to e.n do write(e.c[j])
end;
writeln
end;

procedure xapp(e1: extnum; var e2: extnum; w: word);
begin
if e1.i>w then begin
xstatus:=26;
move(ar0,e2,8)
end
else begin
xtox(e1,e2);
if (e1.n<=w) or (e1.d=0) then exit;
i:=w;
if e2.c[i+1]>5 then begin
if e2.c[i]<9 then inc(e2.c[i])
else begin
while (e2.c[i]=9) and (i>0) do begin
e2.c[i]:=0;
dec(i)
end;
if i>0 then inc(e2.c[i]) else begin
e2.c[w+1]:=0; e2.c[1]:=1;
inc(w);
inc(e2.i)
end
end
end;
e2.d:=w-e2.i;
e2.n:=w;
xelz(e2)
end
end;

procedure xtostr(e: extnum; var s: ansistring);
begin
if e.s=1 then s:='' else s:='-';
for i:=1 to e.i do s:=s+chr(e.c[i]+48);
if e.d>0 then begin
s:=s+'.';
for i:=e.i+1 to e.n do s:=s+chr(e.c[i]+48)
end
end;

procedure strtox(s: ansistring; var e: extnum; var w: word);
begin
if length(s)>maxbuffer then begin
w:=maxbuffer+1;
e.c[1]:=0; e.n:=1; e.i:=1; e.d:=0; e.s:=1;
exit
end;
w:=0;
if s[1]='-' then e.s:=0 else e.s:=1;
if (s[1]='-') or (s[1]='+') then x:=2 else x:=1;
y:=0;
zp:=0;
j:=length(s);
i:=x;
while i<=j do begin
ch:=s[i];
if ch='.' then begin
if (zp>0) or (i=j) then begin
w:=i;
move(ar0,e,8);
exit
end;
if i=x then begin
inc(y);
e.c[y]:=0
end;
zp:=y
end
else if (not (ch in ['0'..'9'])) then begin
w:=i;
move(ar0,e,8);
exit
end
else begin
inc(y);
if y>maxdigits then begin
w:=y;
move(ar0,e,8);
exit
end;
e.c[y]:=ord(ch)-48
end;
inc(i)
end;
e.n:=y;
if zp>0 then begin
e.i:=zp;
e.d:=e.n-e.i
end
else begin
e.i:=y;
e.d:=0
end;
xelz(e)
end;

procedure pitox(var e: extnum);
begin
move(xpg,e,usrdigits+7);
e.n:=usrdigits; e.d:=usrdigits-1
end;

procedure eutox(var e: extnum);
begin
move(xeu,e,usrdigits+7);
e.n:=usrdigits; e.d:=usrdigits-1
end;

procedure zerotox(var e: extnum);
begin
move(ar0,e,8)
end;

procedure onetox(var e: extnum);
begin
move(ar1,e,8)
end;

procedure monetox(var e: extnum);
begin
move(arm1,e,8)
end;

procedure twotox(var e: extnum);
begin
move(ar2,e,8)
end;

procedure etox (ex: extended; var e: extnum);
begin
str(ex,s);
b:=pos('E',s);
if b=0 then begin
xstatus:=1;
move(ar0,e,8);
exit
end;
val(copy(s,b+1,length(s)-b),nd,ip);
s:=copy(s,1,b-1);
if s[1]=' ' then e.s:=1 else e.s:=0;
delete(s,1,1);
while s[length(s)]='0' do delete(s,length(s),1);
delete(s,2,1);
j:=length(s);
if nd=0 then begin
for i:=1 to j do e.c[i]:=ord(s[i])-48;
e.n:=j; e.i:=1; e.d:=j-1
end
else if nd<0 then begin
nd:=-nd;
x:=nd+j;
if x>usrdigits then begin
xstatus:=1;
move(ar0,e,8);
exit
end;
for i:=1 to nd do e.c[i]:=0;
for i:=1 to j do e.c[nd+i]:=ord(s[i])-48;
e.n:=x; e.i:=1; e.d:=x-1
end
else begin
x:=nd+1;
if x>usrdigits then begin
xstatus:=1;
move(ar0,e,8);
exit
end;
for i:=1 to j do e.c[i]:=ord(s[i])-48;
if nd<=j-1 then begin
e.n:=j; e.d:=j-nd-1; e.i:=nd+1
end
else begin
for i:=1 to nd+1-j do e.c[j+i]:=0;
e.n:=x; e.i:=x; e.d:=0
end
end;
xelz(e);
xstatus:=0
end;

procedure xtoe(e: extnum; var ex: extended);
begin
xstatus:=0;
xtox(e,c2);
xelz(c2);
if (c2.c[1]=0) and (c2.n=1) then begin
ex:=0.0;
exit
end;
i:=1;
while (c2.c[i]=0) and (i<c2.n) do inc(i);
dec(i);
if i=0 then nd:=e.i-1 else nd:=-i;
j:=c2.n-i;
if j>16 then y:=17 else y:=j;
if i>0 then for j:=1 to y do c2.c[j]:=c2.c[j+i];
if y>16 then begin
if c2.c[y]>5 then
if c2.c[y-1]<9 then inc(c2.c[y-1])
else begin
j:=y-1;
while (c2.c[j]=9) and (j>0) do begin
c2.c[j]:=0;
dec(j)
end;
if j>0 then inc(c2.c[j])
else begin
inc(nd);
c2.c[1]:=1
end
end;
dec(y)
end
else if y<16 then for j:=y+1 to 16 do c2.c[j]:=0;
s:='';
for j:=1 to 16 do s:=s+chr(c2.c[j]+48);
if e.s=1 then s:=' '+s else s:='-'+s;
insert('.',s,3);
str(abs(nd),s1);
while length(s1)<3 do s1:='0'+s1;
if nd<0 then s1:='E-'+s1 else s1:='E+'+s1;
s:=s+s1;
val(s,ex,ip);
str(ex,s);
b:=pos('E',s);
val(copy(s,b+1,length(s)-b),nd,ip);
if nd<14 then exit;
s:=copy(s,1,b-1);
delete(s,1,1);
while s[length(s)]='0' do delete(s,length(s),1);
delete(s,2,1);
j:=length(s);
for i:=1 to j do c2.c[i]:=ord(s[i])-48;
if nd+1>j then for i:=1 to nd-j+1 do c2.c[j+i]:=0;
j:=nd+1;
if e.i<>j then b:=1
else begin
b:=0;
for i:=1 to j do
if e.c[i]<>c2.c[i] then begin
b:=1;
break
end
end;
if b=1 then begin
xstatus:=2;
ex:=0.0
end
end;

procedure wtox(k: word; var e: extnum);
begin
mpw:=k;
z:=0;
repeat
ac[z]:=mpw mod 10;
inc(z);
mpw:=mpw div 10;
until mpw=0;
for b:=1 to z do e.c[b]:=ac[z-b];
e.n:=z; e.i:=z; e.d:=0; e.s:=1
end;

procedure ltox(l: longint; var e: extnum);
begin
uf:=abs(l);
z:=0;
repeat
ac[z]:=uf mod 10;
uf:=uf div 10;
inc(z);
until uf=0;
for b:=1 to z do e.c[b]:=ac[z-b];
e.n:=z; e.i:=z; e.d:=0;
if l<0 then e.s:=0 else e.s:=1
end;

procedure dwtox(c: longword; var e: extnum);
begin
mpg:=c;
z:=0;
repeat
ac[z]:=mpg mod 10;
inc(z);
mpg:=mpg div 10;
until mpg=0;
for b:=1 to z do e.c[b]:=ac[z-b];
e.n:=z; e.i:=z; e.d:=0; e.s:=1
end;

procedure qwtox(q: qword; var e: extnum);
begin
tb:=q;
z:=0;
repeat
ac[z]:=tb mod 10;
inc(z);
tb:=tb div 10;
until tb=0;
for b:=1 to z do e.c[b]:=ac[z-b];
e.n:=z; e.i:=z; e.d:=0; e.s:=1
end;

function xeq(e1,e2: extnum):boolean;
begin
if (e1.n<>e2.n) or (e1.i<>e2.i) or (e1.s<>e2.s) then xeq:=false
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.n) do inc(i);
xeq:=e1.c[i]=e2.c[i]
end
end;

function xne(e1,e2: extnum):boolean;
begin
if (e1.n<>e2.n) or (e1.i<>e2.i) or (e1.s<>e2.s) then xne:=true
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.n) do inc(i);
if e1.c[i]<>e2.c[i] then xne:=true else xne:=false
end
end;

function xgt(e1,e2: extnum):boolean;
begin
if e1.s<>e2.s then ux:=e1.s
else begin
if e1.i>e2.i then ux:=e1.s
else if e1.i<e2.i then ux:=e1.s xor 1
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.i) do inc(i);
if e1.c[i]>e2.c[i] then ux:=e1.s
else if e1.c[i]<e2.c[i] then ux:=e1.s xor 1
else begin
if (e1.d=0) or (e2.d=0) then begin
if e1.d>0 then ux:=e1.s
else if e2.d>0 then ux:=e1.s xor 1
else ux:=0
end
else begin
i:=e1.i+1;
j:=1;
while (j<e1.d) and (j<e2.d) and (e1.c[i]=e2.c[i]) do begin
inc(i);
inc(j)
end;
if e1.c[i]>e2.c[i] then ux:=e1.s
else if e1.c[i]<e2.c[i] then ux:=e1.s xor 1
else if e1.d>e2.d then ux:=e1.s
else if e1.d<e2.d then ux:=e1.s xor 1
else ux:=0
end
end
end
end;
xgt:=boolean(ux)
end;

function xlt(e1,e2: extnum):boolean;
begin
if e1.s<>e2.s then ux:=e1.s xor 1
else begin
if e1.i<e2.i then ux:=e1.s
else if e1.i>e2.i then ux:=e1.s xor 1
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.i) do inc(i);
if e1.c[i]<e2.c[i] then ux:=e1.s
else if e1.c[i]>e2.c[i] then ux:=e1.s xor 1
else begin
if (e1.d=0) or (e2.d=0) then begin
if e2.d>0 then ux:=e1.s
else if e1.d>0 then ux:=e1.s xor 1
else ux:=0
end
else begin
i:=e1.i+1;
j:=1;
while (j<e1.d) and (j<e2.d) and (e1.c[i]=e2.c[i]) do begin
inc(i);
inc(j)
end;
if e1.c[i]<e2.c[i] then ux:=e1.s
else if e1.c[i]>e2.c[i] then ux:=e1.s xor 1
else if e1.d<e2.d then ux:=e1.s
else if e1.d>e2.d then ux:=e1.s xor 1
else ux:=0
end
end
end
end;
xlt:=boolean(ux)
end;

function xgte(e1,e2: extnum):boolean;
begin
if e1.s<>e2.s then ux:=e1.s
else begin
if e1.i>e2.i then ux:=e1.s
else if e1.i<e2.i then ux:=e1.s xor 1
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.i) do inc(i);
if e1.c[i]>e2.c[i] then ux:=e1.s
else if e1.c[i]<e2.c[i] then ux:=e1.s xor 1
else begin
if (e1.d=0) or (e2.d=0) then begin
if e1.d>0 then ux:=e1.s
else if e2.d>0 then ux:=e1.s xor 1
else ux:=1
end
else begin
i:=e1.i+1;
j:=1;
while (j<e1.d) and (j<e2.d) and (e1.c[i]=e2.c[i]) do begin
inc(i);
inc(j)
end;
if e1.c[i]>e2.c[i] then ux:=e1.s
else if e1.c[i]<e2.c[i] then ux:=e1.s xor 1
else if e1.d>e2.d then ux:=e1.s
else if e1.d<e2.d then ux:=e1.s xor 1
else ux:=1
end
end
end
end;
xgte:=boolean(ux)
end;

function xlte(e1,e2: extnum):boolean;
begin
if e1.s<>e2.s then ux:=e1.s xor 1
else begin
if e1.i<e2.i then ux:=e1.s
else if e1.i>e2.i then ux:=e1.s xor 1
else begin
i:=1;
while (e1.c[i]=e2.c[i]) and (i<e1.i) do inc(i);
if e1.c[i]<e2.c[i] then ux:=e1.s
else if e1.c[i]>e2.c[i] then ux:=e1.s xor 1
else begin
if (e1.d=0) or (e2.d=0) then begin
if e2.d>0 then ux:=e1.s
else if e1.d>0 then ux:=e1.s xor 1
else ux:=1
end
else begin
i:=e1.i+1;
j:=1;
while (j<e1.d) and (j<e2.d) and (e1.c[i]=e2.c[i]) do begin
inc(i);
inc(j)
end;
if e1.c[i]<e2.c[i] then ux:=e1.s
else if e1.c[i]>e2.c[i] then ux:=e1.s xor 1
else if e1.d<e2.d then ux:=e1.s
else if e1.d>e2.d then ux:=e1.s xor 1
else ux:=1
end
end
end
end;
xlte:=boolean(ux)
end;

procedure xfrac(e1: extnum; var e2: extnum);
begin
j:=e1.i;
e2.n:=e1.d+1;
e2.i:=1;
e2.d:=e1.d;
e2.s:=e1.s;
e2.c[1]:=0;
for i:=1 to e1.d do e2.c[i+1]:=e1.c[j+i];
if e2.n=1 then e2.s:=1
end;

procedure xint(e1: extnum; var e2: extnum);
begin
e2.n:=e1.i;
e2.d:=0;
e2.i:=e1.i;
e2.s:=e1.s;
move(e1.c,e2.c,e1.i);
if (e2.n=1) and (e2.c[1]=0) then e2.s:=1
end;

function xsign(e: extnum): longint;
begin
if (e.c[1]=0) and (e.n=1) then xsign:=0
else if e.s=1 then xsign:=1
else xsign:=-1
end;

procedure xceil(e1: extnum; var e2: extnum);
begin
move(e1,e2,e1.n+7);
xelz(e2);
if (e2.s=1) and (e2.d>0) then begin
move(ar1,c2,8);
xadd(e2,c2,e2)
end;
e2.n:=e2.i; e2.d:=0
end;

procedure xfloor(e1: extnum; var e2: extnum);
begin
move(e1.c,e2.c,e1.n+7);
xelz(e2);
if (e2.s=0) and (e2.d>0) then begin
move(arm1,c2,8);
xadd(e2,c2,e2)
end;
e2.n:=e2.i; e2.d:=0
end;

procedure xabs(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
e2.s:=1
end;

procedure xadd(e1,e2:extnum; var e3:extnum);
begin
xstatus:=0;
n3:=e1.n;
n2:=e2.n;
sg1:=e1.s;
if subflag then sg2:=(e2.s xor 1) else sg2:=e2.s;
x:=e1.i;
y:=e2.i;
if x>y then begin
move(e1.c,b1,n3);
dec(x,y);
for i:=1 to n2 do b2[i+x]:=e2.c[i];
fillchar(b2,x,0);
inc(n2,x)
end
else if x<y then begin
move(e2.c,b2,n2);
dec(y,x);
for i:=1 to n3 do b1[i+y]:=e1.c[i];
fillchar(b1,y,0);
inc(n3,y)
end
else begin
move(e1.c,b1,n3);
move(e2.c,b2,n2)
end;
x:=e1.d;
y:=e2.d;
if x>y then begin
da3:=x;
dec(x,y);
for i:=1 to x do b2[n2+i]:=0
end
else if x<y then begin
da3:=y;
dec(y,x);
for i:=1 to y do b1[n3+i]:=0;
inc(n3,y)
end
else da3:=x;
if sg1=sg2 then begin
sg3:=sg1;
z:=0;
for i:=n3 downto 1 do begin
z:=b1[i]+b2[i]+ck[z];
b3[i]:=m10[z]
end;
if z>9 then begin
for i:=n3 downto 1 do b3[i+1]:=b3[i];
b3[1]:=1;
inc(n3)
end
end
else begin
i:=1;
while (b1[i]=b2[i]) and (i<n3) do inc(i);
if b1[i]=b2[i] then begin
move(ar0,e3,8);
exit
end;
if b1[i]>b2[i] then begin
sg3:=sg1;
r:=0;
for i:=n3 downto 1 do begin
r:=b1[i]-b2[i]-cu[r];
b3[i]:=cd[r]
end
end
else begin
sg3:=sg2;
r:=0;
for i:=n3 downto 1 do begin
r:=b2[i]-b1[i]-cu[r];
b3[i]:=cd[r]
end
end
end;
ia3:=n3-da3;
if (ia3>1) and (b3[1]=0) then begin
i:=1;
while (i<ia3) and (b3[i]=0) do inc(i);
if i>1 then begin
dec(i);
dec(n3,i);
dec(ia3,i);
for j:=1 to n3 do b3[j]:=b3[j+i]
end
end;
if n3>usrdigits then
if ia3>usrdigits then begin
xstatus:=5;
move(ar0,e3,8);
exit
end
else begin
n3:=usrdigits;
da3:=n3-ia3
end;
if (b3[n3]=0) and (da3>0) then begin
while (b3[n3]=0) and (n3>ia3) do dec(n3);
da3:=n3-ia3
end;
move(b3,e3.c,n3);
e3.n:=n3; e3.d:=da3; e3.i:=ia3; e3.s:=sg3
end;

procedure xsub(e1,e2: extnum; var e3: extnum);
begin
subflag:=true;
xadd(e1,e2,e3);
if xstatus=5 then xstatus:=6;
subflag:=false
end;

{* the multiplication divides the digits in groups of eight (any group is
stored in a qword) and performs a simplified polynomial calculus of the
product, then converts the qwords of the result to single digits.
This multiplication can work with operands of up to about 14000 digits *}

procedure xmul(e1,e2:extnum; var e3: extnum);
begin
n1:=(e1.n shr 3)+1;
m:=2;
j:=(e1.n and 7)+1;
case j of
8: ub1[1]:=(((((e1.c[1]*10+e1.c[2])*10+e1.c[3])*10+e1.c[4])*10+e1.c[5])*10
+e1.c[6])*10+e1.c[7];
7: ub1[1]:=((((e1.c[1]*10+e1.c[2])*10+e1.c[3])*10+e1.c[4])*10+e1.c[5])*10
+e1.c[6];
6: ub1[1]:=(((e1.c[1]*10+e1.c[2])*10+e1.c[3])*10+e1.c[4])*10+e1.c[5];
5: ub1[1]:=((e1.c[1]*10+e1.c[2])*10+e1.c[3])*10+e1.c[4];
4: ub1[1]:=(e1.c[1]*10+e1.c[2])*10+e1.c[3];
3: ub1[1]:=e1.c[1]*10+e1.c[2];
2: ub1[1]:=e1.c[1];
else begin
dec(m);
dec(n1)
end
end;
for x:=m to n1 do begin
ub1[x]:=((((((e1.c[j]*10+e1.c[j+1])*10+e1.c[j+2])*10+e1.c[j+3])*10+
e1.c[j+4])*10+e1.c[j+5])*10+e1.c[j+6])*10+e1.c[j+7];
inc(j,8)
end;
n2:=(e2.n shr 3)+1;
m:=2;
j:=(e2.n and 7)+1;
case j of
8: ub2[1]:=(((((e2.c[1]*10+e2.c[2])*10+e2.c[3])*10+e2.c[4])*10+e2.c[5])*10
+e2.c[6])*10+e2.c[7];
7: ub2[1]:=((((e2.c[1]*10+e2.c[2])*10+e2.c[3])*10+e2.c[4])*10+e2.c[5])*10
+e2.c[6];
6: ub2[1]:=(((e2.c[1]*10+e2.c[2])*10+e2.c[3])*10+e2.c[4])*10+e2.c[5];
5: ub2[1]:=((e2.c[1]*10+e2.c[2])*10+e2.c[3])*10+e2.c[4];
4: ub2[1]:=(e2.c[1]*10+e2.c[2])*10+e2.c[3];
3: ub2[1]:=e2.c[1]*10+e2.c[2];
2: ub2[1]:=e2.c[1];
else begin
dec(m);
dec(n2)
end
end;
for x:=m to n2 do begin
ub2[x]:=((((((e2.c[j]*10+e2.c[j+1])*10+e2.c[j+2])*10+e2.c[j+3])*10+
e2.c[j+4])*10+e2.c[j+5])*10+e2.c[j+6])*10+e2.c[j+7];
inc(j,8)
end;
n3:=n1+n2;
m:=n1+1;
x:=n2+1;
n:=2;
mpc:=0;
for i:=n3 downto 2 do begin
if i>x then nc:=i-n2 else nc:=1;
if i>m then lv:=n1 else lv:=i-1;
for j:=nc to lv do inc(mpc,ub1[j]*ub2[i-j]);
mpq:=mpc div vcc;
ub3[i]:=mpc-mpq*vcc;
mpc:=mpq
end;
if mpc=0 then while (ub3[n]=0) and (n<n3) do inc(n)
else begin
ub3[1]:=mpc;
dec(n)
end;
xstatus:=0;
q:=lo(ub3[n]);
j:=0;
repeat
k:=q div 10;
ac[j]:=q-k*10;
q:=k;
inc(j);
until q=0;
ncprd:=j+((n3-n) shl 3);
ndprd:=e1.d+e2.d;
if ndprd<ncprd then begin
niprd:=ncprd-ndprd;
ncprd:=0;
if niprd>usrdigits then begin
xstatus:=7;
move(ar0,e3,8);
exit
end
end
else begin
ncprd:=ndprd-ncprd+1;
if ncprd<usrdigits then begin
fillchar(e3.c,ncprd,0);
niprd:=1
end
else begin
move(ar0,e3,8);
exit
end
end;
for i:=1 to j do e3.c[ncprd+i]:=ac[j-i];
inc(ncprd,j);
while (n<n3) and (ncprd<usrdigits) do begin
inc(n);
q:=lo(ub3[n]);
inc(ncprd);
k:=q div p7;
e3.c[ncprd]:=k;
q:=q-k*p7;
inc(ncprd);
k:=q div p6;
e3.c[ncprd]:=k;
q:=q-k*p6;
inc(ncprd);
k:=q div p5;
e3.c[ncprd]:=k;
q:=q-k*p5;
inc(ncprd);
k:=q div p4;
e3.c[ncprd]:=k;
q:=q-k*p4;
inc(ncprd);
k:=q div p3;
e3.c[ncprd]:=k;
q:=q-k*p3;
inc(ncprd);
k:=q div p2;
e3.c[ncprd]:=k;
q:=q-k*p2;
inc(ncprd);
k:=q div p1;
e3.c[ncprd]:=k;
inc(ncprd);
e3.c[ncprd]:=q-k*p1
end;
if ncprd>usrdigits then ncprd:=usrdigits;
ndprd:=ncprd-niprd;
if (e3.c[ncprd]=0) and (ndprd>0) then begin
while (e3.c[ncprd]=0) and (ncprd>niprd) do dec(ncprd);
ndprd:=ncprd-niprd
end;
e3.n:=ncprd; e3.d:=ndprd; e3.i:=niprd;
if (e3.n=1) and (e3.c[1]=0) then e3.s:=1
else
if e1.s=e2.s then e3.s:=1 else e3.s:=0
end;

{* the division operates with the method of progressive partial quotients *}

procedure xdiv(e1,e2: extnum; var e3:extnum);

procedure xdivi;
label cycle1,cycle2;
begin
if sgdr1=sgdr2 then sgqur:=1 else sgqur:=0;
i:=k1-dx;
if b1[k1]=0 then begin
while (b1[k1]=0) and (k1>i) do dec(k1);
dx:=k1-i
end;
j:=1;
while (b1[j]=0) and (j<i) do inc(j);
if j>1 then begin
dec(j);
dec(k1,j);
for p:=1 to k1 do b1[p]:=b1[p+j]
end;
i:=k2-dy;
if b2[k2]=0 then begin
while (b2[k2]=0) and (k2>i) do dec(k2);
dy:=k2-i
end;
j:=1;
while (b2[j]=0) and (j<i) do inc(j);
if j>1 then begin
dec(j);
dec(k2,j);
for p:=1 to k2 do b2[p]:=b2[p+j]
end;
if (b2[1]=0) and (k2=1) then begin
xstatus:=9;
exit
end;
if (b1[1]=0) and (k1=1) then begin
b3[1]:=0; ncqur:=1; ndqur:=0; sgqur:=1;
exit
end;
i:=1;
while (b1[i]=0) and (i<k1) do inc(i);
if i>1 then begin
dec(i);
for j:=1 to k1 do b1[j]:=b1[j+i];
dec(k1,i);
dec(dx,i);
for j:=1 to i do b2[k2+j]:=0;
inc(k2,i)
end;
i:=1;
while (b2[i]=0) and (i<k2) do inc(i);
if i>1 then begin
dec(i);
for j:=1 to k2 do b2[j]:=b2[j+i];
dec(k2,i);
dec(dy,i);
for j:=1 to i do b1[k1+j]:=0;
inc(k1,i)
end;
if dx<dy then begin
i:=dy-dx;
for j:=1 to i do b1[k1+j]:=0;
inc(k1,i)
end
else
if dy<dx then begin
i:=dx-dy;
for j:=1 to i do b2[k2+j]:=0;
inc(k2,i)
end;
ndqur:=0;
ncqur:=0;
if k1<k2 then begin
j:=k2-k1;
for i:=1 to j do begin
b1[k1+i]:=0;
b3[i]:=0
end;
ndqur:=j;
ncqur:=j;
k1:=k2
end;
if k1=k2 then begin
i:=1;
while (b1[i]=b2[i]) and (i<k1) do inc(i);
if b1[i]<b2[i] then begin
inc(k1);
b1[k1]:=0;
inc(ncqur);
b3[ncqur]:=0;
inc(ndqur)
end
end;
for p:=1 to k2 do b7[p+1]:=b2[p];
b7[1]:=0;
ns:=k2+1;
p:=1;
while (b1[p]=b2[p]) and (p<k2) do inc(p);
if b1[p]<b2[p] then nc:=ns else nc:=k2;
move(b1,b5,nc);
ndv:=nc;
if k2<5 then b:=k2 else b:=5;
case b of
1 : md2:=b2[1]+1;
2 : md2:=b2[1]*10+b2[2]+1;
3 : md2:=b2[1]*100+b2[2]*10+b2[3]+1;
4 : md2:=b2[1]*1000+b2[2]*100+b2[3]*10+b2[4]+1;
5 : md2:=b2[1]*10000+b2[2]*1000+b2[3]*100+b2[4]*10+b2[5]+1
end;
lv:=nc;
cycle1:
if lv>k2 then d:=6 else d:=5;
if ndv>d then b:=d else b:=ndv;
case b of
1 : md1:=b5[1];
2 : md1:=b5[1]*10+b5[2];
3 : md1:=b5[1]*100+b5[2]*10+b5[3];
4 : md1:=b5[1]*1000+b5[2]*100+b5[3]*10+b5[4];
5 : md1:=b5[1]*10000+b5[2]*1000+b5[3]*100+b5[4]*10+b5[5];
6 : md1:=b5[1]*100000+b5[2]*10000+b5[3]*1000+b5[4]*100+b5[5]*10+b5[6]
end;
h:=md1 DIV md2;
if h<2 then begin
if lv>k2 then move(b7,b6,ns)
else move(b2,b6,k2);
h:=1;
r:=0;
for p:=lv downto 1 do begin
r:=b5[p]-b6[p]-cu[r];
b4[p]:=cd[r]
end
end
else begin
r1:=0;
r:=0;
m:=lv;
for p:=k2 downto 1 do begin
r1:=b2[p]*h+d10[r1];
r:=b5[m]-m10[r1]-cu[r];
b4[m]:=cd[r];
dec(m)
end;
if m>0 then begin
b4[1]:=cd[b5[1]-d10[r1]-cu[r]];
move(b7,b6,ns)
end
else move(b2,b6,k2)
end;
while (b4[p]=b6[p]) and (p<lv) do inc(p);
while b4[p]>b6[p] do begin
move(b4,b5,nc);
ndv:=nrr;
inc(h);
r:=0;
for p:=lv downto 1 do begin
r:=b5[p]-b6[p]-cu[r];
b4[p]:=cd[r]
end;
while (b4[p]=b6[p]) and (p<lv) do inc(p)
end;
if b4[p]=b6[p] then begin
inc(h);
b4[1]:=0;
nrr:=1
end
else nrr:=lv;
cycle2:
inc(ncqur);
b3[ncqur]:=h;
if ((ncqur>=usrdigits) or ((nrr=1) and (b4[1]=0))) and (ndqur>0) then exit;
inc(nc);
if (nrr=1) and (b4[1]=0) then i:=1
else begin
p:=1;
while (b4[p]=0) and (p<lv) do inc(p);
i:=0;
for m:=p to lv do begin
inc(i);
b5[i]:=b4[m]
end;
inc(i)
end;
if (nc>k1) then begin
b5[i]:=0;
inc(ndqur)
end
else b5[i]:=b1[nc];
ndv:=i;
lv:=i;
if lv<k2 then begin
h:=0;
move(b5,b4,ndv);
nrr:=ndv;
goto cycle2
end;
if lv=k2 then begin
p:=1;
while (b5[p]=b2[p]) and (p<k2) do inc(p);
if (b5[p]<b2[p]) then begin
h:=0;
move(b5,b4,ndv);
nrr:=ndv;
goto cycle2
end
end;
goto cycle1
end;

begin
xstatus:=0;
move(e1.c,b1,e1.n);
k1:=e1.n;
dx:=e1.d;
sgdr1:=e1.s;
move(e2.c,b2,e2.n);
k2:=e2.n;
dy:=e2.d;
sgdr2:=e2.s;
xdivi;
if xstatus=9 then begin
move(ar0,e3,8);
exit
end;
ns:=ncqur-ndqur;
if ns>usrdigits then begin
xstatus:=8;
move(ar0,e3,8);
exit
end;
move(b3,e3.c,ncqur);
e3.s:=sgqur;
e3.n:=ncqur;
e3.d:=ndqur;
e3.i:=ns;
xelz(e3)
end;

procedure xmulq(e1: extnum; q: qword; var e2:extnum);
begin
qwtox(q,c4);
xmul(e1,c4,e2)
end;

procedure xdivq(e1: extnum; q: qword; var e2: extnum);
begin
qwtox(q,c4);
xdiv(e1,c4,e2)
end;

procedure xmul2(e1: extnum; var e2: extnum);
begin
e2.n:=e1.n;
e2.i:=e1.i;
e2.d:=e1.d;
e2.s:=e1.s;
z:=0;
for i:=e2.n downto 1 do begin
d:=(e1.c[i] shl 1)+z;
e2.c[i]:=d mod 10;
z:=d div 10
end;
if (e2.d>0) and (e2.c[e2.n]=0) then begin
dec(e2.d);
dec(e2.n)
end;
if z>0 then begin
for i:=e2.n downto 1 do e2.c[i+1]:=e2.c[i];
e2.c[1]:=z;
inc(e2.n);
inc(e2.i)
end
end;

procedure xmul2s(var e: extnum);
begin
z:=0;
for i:=e.n downto 1 do begin
d:=(e.c[i] shl 1)+z;
e.c[i]:=d mod 10;
z:=d div 10
end;
if (e.d>0) and (e.c[e.n]=0) then begin
dec(e.d);
dec(e.n)
end;
if z>0 then begin
for i:=e.n downto 1 do e.c[i+1]:=e.c[i];
e.c[1]:=z;
inc(e.n);
inc(e.i)
end
end;

procedure xdiv2(e1: extnum; var e2: extnum);
begin
e2.n:=e1.n;
e2.i:=e1.i;
e2.d:=e1.d;
e2.s:=e1.s;
z:=0;
for i:=1 to e1.n do begin
d:=z*10+e1.c[i];
z:=d and 1;
e2.c[i]:=d shr 1
end;
if z>0 then begin
inc(i);
e2.c[i]:=5;
e2.n:=i;
inc(e2.d)
end;
if (e2.i>1) and (e2.c[1]=0) then begin
dec(e2.n);
dec(e2.i);
for i:=1 to e2.n do e2.c[i]:=e2.c[i+1]
end
end;

procedure xdiv2s(var e: extnum);
begin
z:=0;
for i:=1 to e.n do begin
d:=z*10+e.c[i];
z:=d and 1;
e.c[i]:=d shr 1
end;
if z>0 then begin
inc(i);
e.c[i]:=5;
e.n:=i;
inc(e.d)
end;
if (e.i>1) and (e.c[1]=0) then begin
dec(e.n);
dec(e.i);
for i:=1 to e.n do e.c[i]:=e.c[i+1]
end
end;

procedure xinv(e1: extnum; var e2: extnum);
begin
move(ar1,e2,8);
xdiv(e2,e1,e2)
end;

procedure xpow(e1,e2: extnum; var e3: extnum);
begin
xtox(e1,c3);
xtox(e2,c4);
xelz(c4);
xstatus:=0;
move(ar0,c5,8);
if (xgt(c5,c3) and (c4.d>0))
or (xeq(c4,c5) and xeq(c3,c5))
or (xeq(c3,c5) and xgt(c5,c4)) then begin
xstatus:=22;
xtox(c5,e3);
exit
end;
if c4.d>0 then begin
xln(e1,e3);
xmul(e2,e3,e3);
if xstatus=0 then xexp(e3,e3);
if xstatus>0 then begin
xstatus:=23;
xtox(c5,e3)
end;
exit
end;
c5.c[1]:=1;
powsign:=c4.s;
if powsign=0 then begin
sss:=c3.s;
c3.s:=1;
if xgt(c5,c3) then begin
xdiv(c5,c3,c3);
powsign:=1
end;
c3.s:=sss
end;
c4.s:=1;
move(ar2,c6,8);
while (c4.s=1) and ((c4.c[1]>0) or (c4.n>1)) do begin
sss:=c4.c[c4.n] and 1;
xdiv2s(c4);
c4.n:=c4.i; c4.d:=0;
if sss=1 then begin
xmul(c5,c3,c5);
if xstatus>0 then begin
xstatus:=23;
c6.c[1]:=0;
xtox(c6,e3);
exit
end
end;
if (c4.c[1]>0) or (c4.n>1) then begin
xmul(c3,c3,c3);
if xstatus>0 then begin
xstatus:=23;
c6.c[1]:=0;
xtox(c6,e3);
exit
end
end
end;
if powsign=0 then
if (c5.c[1]=0) and (c5.n=1) then begin
xstatus:=23;
c6.c[1]:=0;
xtox(c6,e3);
exit
end
else begin
c6.c[1]:=1;
xdiv(c6,c5,c5)
end;
xtox(c5,e3)
end;

procedure xpow1(e1: extnum; k: longint; var e2: extnum);
begin
xtox(e1,c3);
xelz(c3);
xstatus:=0;
if (c3.c[1]=0) and (c3.n=1) then begin
move(ar0,e2,8);
if k<=0 then xstatus:=24;
exit
end
else if (c3.c[1]=1) and (c3.n=1) then begin
move(ar1,e2,8)
end
else if k=0 then begin
move(ar1,e2,8);
exit
end
else if k=1 then xtox(c3,e2)
else begin
if k<0 then kv:=-k else kv:=k;
mpg:=kv;
move(ar1,e2,8);
while mpg>0 do begin
while mpg and 1=0 do begin
mpg:=mpg shr 1;
xmul(c3,c3,c3);
if xstatus>0 then break
end;
if xstatus>0 then break;
dec(mpg);
xmul(e2,c3,e2);
if xstatus>0 then break
end;
if xstatus>0 then begin
xstatus:=25;
move(ar0,e2,8)
end
end;
if k<0 then begin
xinv(e2,e2);
kv:=-k
end
else kv:=k;
if odd(kv) then e2.s:=e1.s
end;

procedure xsqr(e1: extnum; var e2: extnum);
begin
xmul(e1,e1,e2);
if xstatus>0 then begin
move(ar0,e2,8);
xstatus:=4
end
end;

{* the square root uses the classical digit-by-digit method to achieve
a reliable precision *}

procedure xsqrt(e1: extnum; var e2: extnum);
label ref1,ref2,ref3;
begin
xstatus:=0;
if e1.s=0 then begin
xstatus:=10;
move(ar0,e2,8);
exit
end;
move(e1.c,b2,e1.n);
ndfr:=e1.d;
ncfr:=e1.n;
i:=1;
while (b2[i]=0) and (i<ncfr) do inc(i);
if i>1 then begin
dec(i);
dec(ncfr,i);
for j:=1 to ncfr do b2[j]:=b2[j+i]
end;
if odd(ndfr) then begin
inc(ndfr);
inc(ncfr);
b2[ncfr]:=0
end;
n:=0;
c:=2-(ncfr and 1);
if c=1 then p:=b2[1] else p:=b2[1]*10+b2[2];
hq:=tr[p];
m:=p-hq*hq;
b1[1]:=hq;
ncvr:=1;
if m<10 then begin
b5[1]:=m;
ncrq:=1
end
else begin
b5[1]:=d10[m];
b5[2]:=m10[m];
ncrq:=2
end;
while n<usrdigits do begin
inc(n);
if c<ncfr then begin
b5[ncrq+1]:=b2[c+1];
b5[ncrq+2]:=b2[c+2]
end
else begin
b5[ncrq+1]:=0;
b5[ncrq+2]:=0
end;
inc(ncrq,2);
inc(c,2);
x:=1;
while (b5[x]=0) and (x<ncrq) do inc(x);
if x>1 then begin
dec(x);
dec(ncrq,x);
for m:=1 to ncrq do b5[m]:=b5[m+x]
end;
if b5[1]=0 then begin
inc(ncvr);
b1[ncvr]:=0;
goto ref2
end;
ncmq:=1;
z:=0;
for x:=ncvr downto 1 do begin
z:=b1[x]*2+d10[z];
inc(ncmq);
b3[ncmq]:=m10[z]
end;
if z>9 then begin
inc(ncmq);
b3[ncmq]:=d10[z]
end;
if ncmq<5 then p:=2 else p:=ncmq-2;
d:=ncmq-p;
m2:=b3[p];
j:=0;
while j<d do begin
inc(j);
inc(m2,b3[p+j]*dpot[j])
end;
inc(d);
t:=d+1;
if ncrq<t then z:=ncrq else z:=t;
b:=z;
m1:=b5[b];
while (b>1) do begin
dec(b);
inc(m1,b5[b]*dpot[z-b])
end;
if z>d then begin
j:=m1 div dpot[z-d];
if j>m2 then m1:=j
end;
hq:=(m1+1) div m2;
if hq=0 then begin
inc(ncvr);
b1[ncvr]:=0;
goto ref1
end;
if hq>9 then hq:=9;
ref3:
b3[1]:=hq;
z:=0;
i:=ncmq+1;
for x:=1 to ncmq do begin
z:=b3[x]*hq+d10[z];
dec(i);
b4[i]:=m10[z]
end;
ncpq:=ncmq;
if z>9 then begin
for x:=ncpq downto 1 do b4[x+1]:=b4[x];
b4[1]:=d10[z];
inc(ncpq)
end;
if ncpq>ncrq then begin
dec(hq);
if hq>0 then goto ref3
else begin
inc(ncvr);
b1[ncvr]:=0;
goto ref1
end
end;
if ncpq=ncrq then begin
x:=1;
while (b4[x]=b5[x]) and (x<ncpq) do inc(x);
if b4[x]>b5[x] then begin
dec(hq);
if hq>0 then goto ref3
else begin
inc(ncvr);
b1[ncvr]:=0;
goto ref1
end
end;
if b4[x]=b5[x] then begin
ncrq:=1;
b5[1]:=0;
inc(ncvr);
b1[ncvr]:=hq;
goto ref2
end
end;
inc(ncvr);
b1[ncvr]:=hq;
if ncrq>ncpq then begin
j:=ncrq-ncpq;
for i:=ncpq downto 1 do b4[i+j]:=b4[i];
fillchar(b4,j,0)
end;
r:=0;
for x:=ncrq downto 1 do begin
r:=b5[x]-b4[x]-cu[r];
b5[x]:=cd[r]
end;
ref1:
x:=1;
while (b5[x]=0) and (x<ncrq) do inc(x);
if x>1 then begin
dec(x);
dec(ncrq,x);
for m:=1 to ncrq do b5[m]:=b5[m+x]
end;
ref2:
end;
i:=e1.i;
if i>1 then begin
i:=(i+1) div 2;
ndvr:=ncvr-i
end
else begin
if e1.d>ncfr then begin
i:=((ndfr-ncfr) div 2)+1;
for j:=ncvr downto 1 do b1[j+i]:=b1[j];
fillchar(b1,i,0);
inc(ncvr,i);
ndvr:=ncvr-1
end
else begin
if e1.c[1]=0 then begin
for j:=ncvr downto 1 do b1[j+1]:=b1[j];
b1[1]:=0;
inc(ncvr)
end;
ndvr:=ncvr-1
end
end;
j:=ncvr-ndvr;
if (j>1) and (b1[1]=0) then begin
i:=1;
while (i<j) and (b1[i]=0) do inc(i);
if i>1 then begin
dec(i);
dec(ncvr,i);
dec(j,i);
for x:=1 to ncvr do b1[x]:=b1[x+i]
end
end;
if (ncvr>usrdigits) and (j<=usrdigits) then begin
ncvr:=usrdigits;
ndvr:=ncvr-j
end;
if (b1[ncvr]=0) and (ndvr>0) then begin
while (b1[ncvr]=0) and (ncvr>j) do dec(ncvr);
ndvr:=ncvr-j
end;
move(b1,e2.c,ncvr);
e2.n:=ncvr; e2.d:=ndvr; e2.i:=j; e2.s:=1
end;

procedure xfact(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
xelz(e2);
if (e2.s=0) or (e2.d>0) then begin
move(ar0,e2,8);
xstatus:=36;
exit
end;
move(ar1,c3,8);
move(ar0,c4,8);
xstatus:=0;
while xgt(e2,c4) and (xstatus=0) do begin
xaddb(c4,1);
xmul(c3,c4,c3)
end;
if xstatus>0 then begin
xstatus:=37;
move(ar0,e2,8)
end
else xtox(c3,e2)
end;

procedure xdfact(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
xelz(e2);
if (e2.s=0) or (e2.d>0) or (e2.c[e2.i] and 1=0) then begin
move(ar0,e2,8);
xstatus:=38;
exit
end;
move(ar1,c3,8);
move(ar1,c4,8);
xstatus:=0;
while xgt(e2,c4) and (xstatus=0) do begin
xaddb(c4,2);
xmul(c3,c4,c3)
end;
if xstatus>0 then begin
xstatus:=39;
move(ar0,e2,8)
end
else xtox(c3,e2)
end;

procedure xdtor(e1: extnum; var e2: extnum);
begin
move(xpg,c2,usrdigits+7);
c2.n:=usrdigits; c2.d:=usrdigits-1;
move(ar180,c3,10);
xdiv(c2,c3,c2);
xmul(e1,c2,e2)
end;

procedure xrtod(e1: extnum; var e2: extnum);
begin
move(xpg,c2,usrdigits+7);
c2.n:=usrdigits; c2.d:=usrdigits-1;
move(ar180,c3,10);
xdiv(c3,c2,c2);
xmul(e1,c2,e2);
if xstatus>0 then xstatus:=3
end;

{* the exponential is calculated by e^x = e^int(x)*e^frac(x).
e^int(x) is evaluated as a power with integer exponent. frac(x) is reduced
below 0.3 by repeated square roots and then e^frac(x) is evaluated by the
Mac Laurin series exp(x) = 1 + x/1! + x^2/2! + x^3/3! + x^4/4! + ...
The maximum value of int(x) must be 11786 because over this limit there is
an overflow due to more than 5119 integer digits in the result *}

procedure xexp(e1: extnum; var e2: extnum);
begin
xstatus:=0;
xtox(e1,c2);
c2.s:=1;
xelz(c2);
if c2.n=1 then begin
if c2.c[1]=0 then begin
move(ar1,e2,8);
exit
end;
if c2.c[1]=1 then begin
move(xeu,e2,usrdigits+7);
e2.d:=usrdigits-1; e2.n:=usrdigits;
if e1.s=0 then begin
move(ar1,c4,8);
xdiv(c4,e2,e2)
end;
exit
end
end;
if c2.c[1]>0 then begin
kr:=1;
qr:=c2.c[c2.i];
for i:=c2.i-1 downto 1 do begin
kr:=10*kr;
qr:=qr+c2.c[i]*kr
end;
if qr>11786 then begin
move(ar0,e2,8);
if c2.s=1 then xstatus:=19 else xstatus:=0;
exit
end;
move(xeu,c3,usrdigits+7);
c3.d:=usrdigits-1; c3.n:=usrdigits;
xtox(c3,e2);
dec(qr);
while qr>0 do begin
if qr and 1 = 1 then begin
xmul(e2,c3,e2);
if xstatus>0 then break
end;
qr:=qr shr 1;
if qr>0 then begin
xmul(c3,c3,c3);
if xstatus>0 then break
end
end;
if xstatus>0 then begin
move(ar0,e2,8);
if c2.s=1 then xstatus:=19 else xstatus:=0;
exit
end
end;
if c2.d>0 then begin
j:=c2.i;
c4.n:=c2.d+1;
c4.i:=1;
c4.d:=c2.d;
c4.s:=1;
c4.c[1]:=0;
for i:=1 to c2.d do c4.c[i+1]:=c2.c[j+i];
inc(usrdigits,5);
move(ar0d3,c5,9);
mpw:=0;
while xgte(c4,c5) do begin
xdiv2s(c4);
inc(mpw)
end;
move(ar1,c5,8);
move(ar1,c6,8);
uf:=0;
repeat
inc(uf);
recip;
xmul(c6,c4,c6);
xmul(c6,rc,c6);
xadd(c5,c6,c5);
until (c6.n=1) and (c6.c[1]=0);
while mpw>0 do begin
xmul(c5,c5,c5);
dec(mpw)
end;
dec(usrdigits,5)
end;
if c2.c[1]>0 then begin
if c2.d>0 then begin
xmul(e2,c5,e2);
if xstatus>0 then begin
move(ar0,e2,8);
if c2.s=1 then xstatus:=19 else xstatus:=0;
exit
end
end
end
else xtox(c5,e2);
if e1.s=0 then begin
move(ar1,c2,8);
xdiv(c2,e2,e2)
end
end;

{* the natural logarithm is calculated by the Taylor series
ln(x) = 2 * (a + a^3/3 + a^5/5 + a^7/7 + a^9/9 + ...) where a = (x-1)/(x+1).
The argument x is reduced below 1.5 by repeated square roots, so (x-1)/(x+1)
before the series is reduced below 0.2. *}

procedure xln(e1: extnum; var e2: extnum);
begin
move(ar0,c7,8);
if xgt(e1,c7) then xstatus:=0
else begin
xstatus:=20;
xtox(c7,e2);
exit
end;
xtox(e1,c4);
xelz(c4);
if (c4.c[1]=1) and (c4.n=1) then begin
move(ar0,e2,8);
exit
end;
if c4.c[1]=0 then begin
np:=1;
i:=c4.n;
while (c4.c[np]=0) and (np<i) do inc(np);
dec(np);
dec(c4.n,np); c4.i:=1; c4.d:=c4.n-1;
for i:=1 to e1.n do c4.c[i]:=c4.c[i+np];
sss:=0
end
else begin
sss:=1;
np:=c4.i-1;
c4.i:=1;
c4.d:=c4.d+np
end;
inc(usrdigits,5);
move(ar1d5,c5,9);
mpw:=1;
while xgte(c4,c5) do begin
xsqrt(c4,c4);
mpw:=mpw shl 1
end;
c7.c[1]:=1;
xadd(c4,c7,c5);
xsub(c4,c7,c6);
xdiv(c6,c5,c4);
xtox(c4,c5);
uf:=1;
xmul(c4,c4,c6);
repeat
inc(uf,2);
recip;
xmul(c4,c6,c4);
xmul(c4,rc,c3);
xadd(c5,c3,c5);
until (c3.n=1) and (c3.c[1]=0);
dec(usrdigits,5);
xmul2s(c5);
if mpw>1 then begin
j:=0;
repeat
i:=mpw div 10;
ac[j]:=mpw-i*10;
mpw:=i;
inc(j);
until mpw=0;
for i:=1 to j do c7.c[i]:=ac[j-i];
c7.n:=j; c7.i:=j; c7.d:=0; c7.s:=1;
xmul(c5,c7,c5)
end;
if np>0 then begin
move(xl10,c7,usrdigits+7);
c7.d:=usrdigits-1; c7.n:=usrdigits;
j:=0;
repeat
ac[j]:=np mod 10;
np:=np div 10;
inc(j);
until np=0;
for i:=1 to j do c4.c[i]:=ac[j-i];
c4.n:=j; c4.i:=j; c4.d:=0; c4.s:=1;
xmul(c7,c4,c7);
if sss=1 then xadd(c5,c7,c5) else xsub(c5,c7,c5)
end;
c5.s:=sss;
xtox(c5,e2)
end;

{* log(x) = ln(x)/ln(10) *}

procedure xlog(e1: extnum; var e2: extnum);
begin
xln(e1,e2);
if xstatus>0 then xstatus:=21
else begin
move(xl10,c3,usrdigits+7);
c3.d:=usrdigits-1; c3.n:=usrdigits;
xdiv(e2,c3,e2)
end
end;

{* To calculate the sine the argument x is reduced from 0 to pi/2,
and then by repeated divisions by 3 is reduced under 0.5. So the Taylor
series sin(x) = x - x^3/3! + x^5/5! - x^7/7! + ... converges more quickly.
Then the sin(x) value is updated by a repeated use of the trisection
identity sin(3x)=3sin(x)-4sin(x)^3 *}

procedure xsin(e1: extnum; var e2: extnum);
begin
xtox(e1,c3);
c3.s:=1;
xelz(c3);
if (c3.n=1) and (c3.c[1]=0) then begin
move(ar0,e2,8);
exit
end;
move(xpg,c6,usrdigits+7);
c6.d:=usrdigits-1; c6.n:=usrdigits;
xmul2(c6,c7);
xdiv(c3,c7,c5);
xfrac(c5,c5);
xmul(c5,c7,c4);
move(ar0d75,c3,10);
if xgt(c5,c3) then begin
xsub(c7,c4,c2);
sss:=0;
qdr:=4
end
else begin
c3.c[2]:=5; c3.n:=2; c3.d:=1;
if xgt(c5,c3) then begin
xsub(c4,c6,c2);
sss:=0;
qdr:=3
end
else begin
c3.c[2]:=2; c3.c[3]:=5; c3.n:=3; c3.d:=2;
if xgt(c5,c3) then begin
xsub(c6,c4,c2);
sss:=1;
qdr:=2
end
else begin
xtox(c4,c2);
sss:=1;
qdr:=1
end
end
end;
move(ar0d5,c5,9);
np:=0;
while xgte(c2,c5) do begin
inc(np);
xdivbs(c2,3)
end;
xtox(c2,e2);
xtox(c2,c6);
xmul(c2,c2,c7);
df:=1;
repeat
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c7,c6);
xsub(e2,c6,e2);
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c7,c6);
xadd(e2,c6,e2);
until (c6.n=1) and (c6.c[1]=0);
if np>0 then begin
move(ar3,c3,8);
while np>0 do begin
xmul(e2,e2,c7);
xmulbs(c7,4);
xsub(c3,c7,c7);
xmul(e2,c7,e2);
dec(np)
end
end;
e2.s:=sss;
if e1.s=0 then e2.s:=e2.s xor 1
end;

{* As for the sine, to calculate the cosine the argument x is reduced
from 0 to pi/2, and then by repeated divisions by 3 is reduced under 0.5.
So the Taylor series 1 - x^2/2! + x^4/4! - x^6/6! + x^8/8! - x^10/10! + ...
can converge more quickly. Then the cos(x) value is updated by a repeated
use of the trisection identity cos(3x)=4cos(x)^3-3cos(x) *}

procedure xcos(e1: extnum; var e2: extnum);
begin
xtox(e1,c3);
c3.s:=1;
xelz(c3);
if (c3.n=1) and (c3.c[1]=0) then begin
move(ar1,e2,8);
exit
end;
move(xpg,c6,usrdigits+7);
c6.d:=usrdigits-1; c6.n:=usrdigits;
xmul2(c6,c7);
xdiv(c3,c7,c5);
xfrac(c5,c5);
xmul(c5,c7,c4);
move(ar0d75,c3,10);
if xgt(c5,c3) then begin
xsub(c7,c4,c2);
sss:=1;
qdr:=4
end
else begin
c3.c[2]:=5; c3.n:=2; c3.d:=1;
if xgt(c5,c3) then begin
xsub(c4,c6,c2);
sss:=0;
qdr:=3
end
else begin
c3.c[2]:=2; c3.c[3]:=5; c3.n:=3; c3.d:=2;
if xgt(c5,c3) then begin
xsub(c6,c4,c2);
sss:=0;
qdr:=2
end
else begin
xtox(c4,c2);
sss:=1;
qdr:=1
end
end
end;
move(ar0d5,c5,9);
np:=0;
while xgte(c2,c5) do begin
inc(np);
xdivbs(c2,3)
end;
move(ar1,e2,8);
move(ar1,c6,8);
xmul(c2,c2,c7);
df:=0;
repeat
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c7,c6);
xsub(e2,c6,e2);
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c7,c6);
xadd(e2,c6,e2);
until (c6.n=1) and (c6.c[1]=0);
if np>0 then begin
move(ar3,c3,8);
while np>0 do begin
xmul(e2,e2,c7);
xmulbs(c7,4);
xsub(c7,c3,c7);
xmul(e2,c7,e2);
dec(np)
end
end;
e2.s:=sss
end;

{* tan(x) = sin(x)/sqrt(1-sin(x)^2). The sign depends on the quadrant *}

procedure xtan(e1: extnum; var e2: extnum);
begin
inc(usrdigits,5);
xsin(e1,e2);
xmul(e2,e2,c4);
if xstatus>0 then begin
xstatus:=11;
move(ar0,e2,8);
dec(usrdigits,5);
exit
end;
move(ar1,c5,8);
xsub(c5,c4,c6);
xsqrt(c6,c5);
dec(usrdigits,5);
xdiv(e2,c5,e2);
if xstatus>0 then begin
xstatus:=11;
move(ar0,e2,8);
exit
end;
if (qdr=1) or (qdr=3) then e2.s:=1 else e2.s:=0;
if e1.s=0 then e2.s:=e2.s xor 1
end;

{* cot(x) = cos(x)/sqrt(1-cos(x)^2). The sign depends on the quadrant *}

procedure xcot(e1: extnum; var e2: extnum);
begin
inc(usrdigits,5);
xcos(e1,e2);
xmul(e2,e2,c4);
if xstatus>0 then begin
xstatus:=12;
move(ar0,e2,8);
dec(usrdigits,5);
exit
end;
move(ar1,c5,8);
xsub(c5,c4,c6);
xsqrt(c6,c5);
dec(usrdigits,5);
xdiv(e2,c5,e2);
if xstatus>0 then begin
xstatus:=12;
move(ar0,e2,8);
exit
end;
if (qdr=1) or (qdr=3) then e2.s:=1 else e2.s:=0;
if e1.s=0 then e2.s:=e2.s xor 1
end;

{* sec(x) = 1/cos(x) *}

procedure xsec(e1: extnum; var e2: extnum);
begin
xcos(e1,e2);
move(ar1,c4,8);
xdiv(c4,e2,e2);
if xstatus>0 then begin
xstatus:=13;
move(ar0,e2,8)
end
end;

{* csc(x) = 1/sin(x) *}

procedure xcsc(e1: extnum; var e2: extnum);
begin
xsin(e1,e2);
move(ar1,c4,8);
xdiv(c4,e2,e2);
if xstatus>0 then begin
xstatus:=14;
move(ar0,e2,8)
end
end;

{* The arcsin(x) if x > 0.447 is evaluated by arcsin(x)=arctan(x/sqrt(1-x*x))
or otherwise by the Taylor series:
arcsin(x)=x+(1)*x^3)/((2)*3)+(1*3)*x^5/((2*4)*5)+(1*3*5)*x^7/((2*4*6)*7)+... *}

procedure xasin(e1: extnum; var e2: extnum);
begin
xtox(e1,c2);
xelz(c2);
c2.s:=1;
move(ar1,c3,8);
if xgt(c2,c3) then begin
xstatus:=15;
move(ar0,e2,8);
exit
end;
if c2.n=1 then begin
if c2.c[1]=0 then begin
move(ar0,e2,8);
exit
end;
if c2.c[1]=1 then begin
move(xpg,e2,usrdigits+7);
e2.d:=usrdigits-1; e2.n:=usrdigits;
xdiv2s(e2);
exit
end
end;
c2.s:=1;
move(ar0d447,c7,11);
if xgt(c2,c7) then begin
xmul(c2,c2,c3);
move(ar1,c4,8);
xsub(c4,c3,c4);
xsqrt(c4,c4);
xdiv(c2,c4,c4);
xatan(c4,e2)
end
else begin
inc(usrdigits,2);
xmul(c2,c2,c5);
xmul(c5,c2,c3);
xdivb(c3,6,c4);
xadd(c2,c4,e2);
xdiv2(c3,c6);
uf:=3;
ef:=1;
repeat
inc(uf);
inc(ef,2);
xmul(c6,c5,c6);
quot;
xmul(c6,rc,c6);
inc(uf);
recip;
xmul(c6,rc,c4);
xadd(e2,c4,e2);
until (c4.n=1) and (c4.c[1]=0);
dec(usrdigits,2);
xapp(e2,e2,usrdigits)
end;
e2.s:=e1.s
end;

{* arccos(x) = pi/2 - arcsin(x) *}

procedure xacos(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
xelz(e2);
e2.s:=1;
move(ar1,c3,8);
if xgt(e2,c3) then begin
xstatus:=16;
move(ar0,e2,8);
exit
end;
if e2.n=1 then begin
if e2.c[1]=0 then begin
move(xpg,e2,usrdigits+7);
e2.d:=usrdigits-1; e2.n:=usrdigits;
xdiv2s(e2);
exit
end;
if e2.c[1]=1 then begin
if e1.s=1 then begin
move(ar0,e2,8);
exit
end
else begin
move(xpg,e2,usrdigits+7);
e2.d:=usrdigits-1; e2.n:=usrdigits;
exit
end
end
end;
xasin(e2,e2);
move(xpg,c3,usrdigits+7);
c3.d:=usrdigits-1; c3.n:=usrdigits;
xdiv2(c3,c4);
xsub(c4,e2,e2);
if e1.s=0 then xsub(c3,e2,e2)
end;

{* the inverse trigonometric tangent is calculated for x < 5.4 and depending
on the operand value by using the Mac Laurin series:
arctan(x) = x - x^3/3! + x^5/5! - x^7/7! + x^9/9! - x^11/11! + ...
or arctan(x) = (x/(1+x^2))*(1 + (2/3)*z + (2*4)*z^2/(3*5) + ...)
where z = x^2/(x^2+1), with some reductions.
For x >= 5.4 is calculated by pi/2-arccot(x) and the MacLaurin series
of the arccotangent 1/x - 1/(3*x^3) + 1/(5*x^5) - 1/(7*x^7) + ... *}

procedure xatan(e1: extnum; var e2: extnum);
begin
xtox(e1,c3);
xelz(c3);
if (c3.n=1) and (c3.c[1]=0) then begin
xtox(c3,e2);
exit
end;
c3.s:=1;
inc(usrdigits);
move(ar5d4,c5,9);
if xgte(c3,c5) then begin
move(ar1,c5,8);
xdiv(c5,c3,c4);
uf:=1;
xtox(c4,e2);
xtox(c4,c6);
xmul(c4,c4,c7);
repeat
inc(uf,2);
recip;
xmul(c6,c7,c6);
xmul(c6,rc,c5);
xsub(e2,c5,e2);
inc(uf,2);
recip;
xmul(c6,c7,c6);
xmul(c6,rc,c5);
xadd(e2,c5,e2);
until (c5.n=1) and (c5.c[1]=0);
move(xpg,c5,usrdigits+7);
c5.n:=usrdigits; c5.d:=usrdigits-1;
xdiv2s(c5);
xsub(c5,e2,e2)
end
else begin
move(ar1,c5,8);
move(ar2,c6,8);
move(ar3,c7,8);
if xgt(c3,c5) then begin
if xgte(c3,c6) and xlt(c3,c7) then begin
sss:=2;
xsqrt(c7,c7);
xmul(c3,c7,c4);
xaddb(c4,1);
xsub(c3,c7,c7);
xdiv(c7,c4,c3)
end
else begin
sss:=1;
xsub(c3,c5,c4);
xadd(c3,c5,c3);
xdiv(c4,c3,c3)
end
end
else sss:=0;
move(ar0d5,c5,9);
if xlt(c3,c5) or (sss=2) then begin
xtox(c3,e2);
uf:=1;
xmul(c3,c3,c6);
xtox(c3,c4);
xtox(c3,c2);
repeat
inc(uf,2);
recip;
xmul(c4,c6,c4);
xmul(c4,rc,c3);
xsub(e2,c3,e2);
inc(uf,2);
recip;
xmul(c4,c6,c4);
xmul(c4,rc,c3);
xadd(e2,c3,e2);
until (c3.n=1) and (c3.c[1]=0);
if sss=2 then begin
move(xpg,c4,usrdigits+7);
c4.d:=usrdigits-1; c4.n:=usrdigits;
xdivbs(c4,3);
xadd(e2,c4,e2)
end
end
else begin
move(ar3,c5,8);
move(arm1,c4,8);
xsqrt(c5,c5);
xmul(c3,c5,c6);
xadd(c6,c4,c6);
xadd(c3,c5,c5);
xdiv(c6,c5,c3);
c4.s:=1;
xmul(c3,c3,c5);
xadd(c5,c4,c7);
xdiv(c5,c7,c5);
xtox(c4,e2);
ef:=0;
uf:=1;
repeat
inc(ef,2);
inc(uf,2);
quot;
xmul(c4,c5,c4);
xmul(c4,rc,c4);
xadd(e2,c4,e2);
until (c4.n=1) and (c4.c[1]=0);
xmul(e2,c3,e2);
xdiv(e2,c7,e2);
move(xpg,c4,usrdigits+7);
c4.d:=usrdigits-1; c4.n:=usrdigits;
xdivbs(c4,6);
xadd(e2,c4,e2)
end;
if sss=1 then begin
move(xpg,c4,usrdigits+7);
c4.d:=usrdigits-1; c4.n:=usrdigits;
xdivbs(c4,4);
xadd(e2,c4,e2)
end
end;
dec(usrdigits);
if (e2.n>usrdigits) and (e2.d>0) then begin
e2.n:=usrdigits;
e2.d:=e2.n-e2.i
end;
xelz(e2);
e2.s:=e1.s
end;

{* arccot(x) = pi/2 - arctan(x) *}

procedure xacot(e1: extnum; var e2: extnum);
begin
xatan(e1,e2);
move(xpg,c4,usrdigits+7);
c4.d:=usrdigits-1; c4.n:=usrdigits;
xdiv2s(c4);
xsub(c4,e2,e2)
end;

{* arcsec(x) = arccos(1/x) *}

procedure xasec(e1: extnum; var e2: extnum);
begin
xstatus:=0;
move(arm1,c3,8);
move(ar1,c4,8);
if xgt(e1,c3) and xgt(c4,e1) then begin
xstatus:=17;
move(ar0,e2,8);
exit
end;
xdiv(c4,e1,e2);
xacos(e2,e2)
end;

{* arccsc(x) = arcsin(1/x) *}

procedure xacsc(e1: extnum; var e2: extnum);
begin
xstatus:=0;
move(arm1,c3,8);
move(ar1,c4,8);
if xgt(e1,c3) and xgt(c4,e1) then begin
xstatus:=18;
move(ar0,e2,8);
exit
end;
xdiv(c4,e1,e2);
xasin(e2,e2)
end;

{* The hyperbolic sine if x is integer or > 14 is calculated by
sinh(x)=(exp(x)-exp(-x))/2. Otherwise, the argument x is reduced under 0.5
by repeated divisions by 3 for a better convergence of the Taylor series
sinh(x) = x + x^3/3! + x^5/5! + x^7/7! + ... then the sinh(x) value is updated
by a repeated use of the identity sinh(3x)=sinh(x)(3+4sinh(x)^2).
The argument x must be < 11788 because over this limit there is an overflow
due to more than 5119 integer digits on the result *}

procedure xsinh(e1: extnum; var e2: extnum);
begin
xtox(e1,c7);
c7.s:=1;
xelz(c7);
if (c7.n=1) and (c7.c[1]=0) then begin
move(ar0,e2,8);
exit
end;
if c7.i>5 then begin
xstatus:=27;
move(ar0,e2,8);
exit
end;
if c7.i=5 then begin
kr:=1;
qr:=c7.c[5];
for i:=4 downto 1 do begin
kr:=10*kr;
qr:=qr+c7.c[i]*kr
end;
if qr>11787 then begin
xstatus:=27;
move(ar0,e2,8);
exit
end
end;
move(ar14,c5,9);
if (c7.d=0) or xgt(c7,c5) then begin
xexp(c7,c7);
if xstatus>0 then begin
xstatus:=27;
move(ar0,e2,8);
exit
end;
move(ar1,c5,8);
xdiv(c5,c7,c2);
xsub(c7,c2,e2);
xdiv2s(e2)
end
else begin
move(ar0d5,c5,9);
inc(usrdigits,5);
np:=0;
while xgte(c7,c5) do begin
inc(np);
xdivbs(c7,3)
end;
xtox(c7,e2);
xtox(c7,c6);
xmul(c7,c7,c2);
df:=1;
repeat
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c2,c6);
xadd(e2,c6,e2);
until (c6.n=1) and (c6.c[1]=0);
if np>0 then begin
move(ar3,c3,8);
while np>0 do begin
xmul(e2,e2,c2);
if xstatus>0 then break;
xmulbs(c2,4);
if xstatus>0 then break;
xadd(c3,c2,c2);
xmul(e2,c2,e2);
if xstatus>0 then break;
dec(np)
end;
if xstatus>0 then begin
xstatus:=27;
move(ar0,e2,8);
dec(usrdigits,5);
exit
end
end;
dec(usrdigits,5);
xapp(e2,e2,usrdigits);
if xstatus=26 then begin
xstatus:=27;
move(ar0,e2,8);
exit
end
end;
e2.s:=e1.s
end;

{* The hyperbolic cosine if x is integer or > 13.5 is calculated by
cosh(x)=(exp(x)+exp(-x))/2. Otherwise, the argument x is reduced under 0.5
by repeated divisions by 3 for a better convergence of the Taylor series
cosh(x) = 1 + x^2/2! + x^4/4! + x^6/6! + ... then the cosh(x) value is updated
by a repeated use of the identity cosh(3x)=cosh(x)(4cosh(x)^2-3).
The argument x must be < 11788 because over this limit there is an overflow
due to more than 5119 integer digits on the result *}

procedure xcosh(e1: extnum; var e2: extnum);
begin
xtox(e1,c7);
c7.s:=1;
xelz(c7);
if (c7.n=1) and (c7.c[1]=0) then begin
move(ar1,e2,8);
exit
end;
if c7.i>5 then begin
xstatus:=28;
move(ar0,e2,8);
exit
end;
if c7.i=5 then begin
kr:=1;
qr:=c7.c[5];
for i:=4 downto 1 do begin
kr:=10*kr;
qr:=qr+c7.c[i]*kr
end;
if qr>11787 then begin
xstatus:=28;
move(ar0,e2,8);
exit
end
end;
move(ar13d5,c5,10);
if (c7.d=0) or xgt(c7,c5) then begin
xexp(c7,c7);
if xstatus>0 then begin
xstatus:=28;
move(ar0,e2,8);
exit
end;
move(ar1,c5,8);
xdiv(c5,c7,c2);
xadd(c7,c2,e2);
xdiv2s(e2)
end
else begin
move(ar0d5,c5,9);
inc(usrdigits,5);
np:=0;
while xgte(c7,c5) do begin
inc(np);
xdivbs(c7,3)
end;
move(ar1,e2,8);
move(ar1,c6,8);
xmul(c7,c7,c2);
df:=0;
repeat
inc(df,2);
uf:=df*(df-1);
recip;
xmul(c6,rc,c6);
xmul(c6,c2,c6);
xadd(e2,c6,e2);
until (c6.n=1) and (c6.c[1]=0);
if np>0 then begin
move(ar3,c3,8);
while np>0 do begin
xmul(e2,e2,c2);
if xstatus>0 then break;
xmulbs(c2,4);
if xstatus>0 then break;
xsub(c2,c3,c2);
xmul(e2,c2,e2);
if xstatus>0 then break;
dec(np)
end;
if xstatus>0 then begin
xstatus:=28;
move(ar0,e2,8);
dec(usrdigits,5);
exit
end
end;
dec(usrdigits,5);
xapp(e2,e2,usrdigits);
if xstatus=26 then begin
xstatus:=28;
move(ar0,e2,8);
exit
end
end
end;

{* tanh(x) = (exp(x)-1/exp(x))/(exp(x)+1/exp(x)) *}

procedure xtanh(e1: extnum; var e2: extnum);
begin
xtox(e1,c7);
xelz(c7);
c7.s:=1;
move(ar1,e2,8);
xexp(c7,c7);
if xstatus>0 then begin
xstatus:=0;
e2.s:=e1.s;
exit
end;
xdiv(e2,c7,c3);
xsub(c7,c3,c4);
xadd(c7,c3,c3);
xdiv(c4,c3,e2);
e2.s:=e1.s
end;

{* coth(x) = sinh(x)/cosh(x) = (exp(x)+1/exp(x))/(exp(x)-1/exp(x)) *}

procedure xcoth(e1: extnum; var e2: extnum);
begin
xtox(e1,c7);
xelz(c7);
if (c7.c[1]=0) and (c7.n=1) then begin
move(ar0,e2,8);
xstatus:=29;
exit
end;
c7.s:=1;
move(ar1,e2,8);
xexp(c7,c7);
if xstatus>0 then begin
xstatus:=0;
e2.s:=e1.s;
exit
end;
xdiv(e2,c7,c3);
xadd(c7,c3,c4);
xsub(c7,c3,c3);
xdiv(c4,c3,e2);
e2.s:=e1.s
end;

{* sech(x) = 1/cosh(x) = 2/(exp(x)+1/exp(x)) *}

procedure xsech(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
e2.s:=1;
xexp(e2,e2);
if xstatus>0 then begin
xstatus:=0;
move(ar0,e2,8);
exit
end;
move(ar1,c2,8);
xdiv(c2,e2,c3);
xadd(e2,c3,c3);
c2.c[1]:=2;
xdiv(c2,c3,e2)
end;

{* csch(x) = 1/sinh(x) =  2/(exp(x)-1/exp(x)) *}

procedure xcsch(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
xelz(e2);
if (e2.c[1]=0) and (e2.n=1) then begin
xstatus:=30;
exit
end;
e2.s:=1;
xexp(e2,e2);
if xstatus>0 then begin
xstatus:=0;
move(ar0,e2,8);
exit
end;
move(ar1,c2,8);
xdiv(c2,e2,c3);
xsub(e2,c3,c3);
c2.c[1]:=2;
xdiv(c2,c3,e2);
e2.s:=e1.s
end;

{* arsinh(x) = ln(x+sqrt(x^2+1)) *}

procedure xasinh(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
e2.s:=1;
xmul(e2,e2,c2);
move(ar1,c3,8);
xadd(c2,c3,c2);
xsqrt(c2,c2);
xadd(c2,e2,c2);
xln(c2,e2);
e2.s:=e1.s
end;

{* arcosh(x) = ln(x+sqrt(x^2-1)) *}

procedure xacosh(e1: extnum; var e2: extnum);
begin
move(ar1,c3,8);
if xlt(e1,c3) then begin
xstatus:=31;
move(ar0,e2,8);
exit
end;
xmul(e1,e1,c2);
xsub(c2,c3,c2);
xsqrt(c2,c2);
xadd(e1,c2,c2);
xln(c2,e2)
end;

{* artanh(x) is calculated by 0.5*(ln(1+x)-ln(1-x)) if x < 0.69. If x >= 0.69
it is evaluated by the Taylor series artanh(x) = x + x^3/3 + x^5/5 + ...
with a previous reduction. The reduction uses the bisection identity
artanh(x)=2*artanh(x/(sqrt(1-x^2)+1)). The x value is reduced
by x:=x/(sqrt(1-x^2)+1) repeated np times until x < 0.5, and later the series
value is updated by a multiplication by 2^np *}

procedure xatanh(e1: extnum; var e2: extnum);
begin
xtox(e1,c2);
c2.s:=1;
move(ar1,c3,8);
if xgte(c2,c3) then begin
xstatus:=32;
move(ar0,e2,8);
exit
end;
move(ar0d69,c5,10);
if xgte(c2,c5) then begin
xsub(c3,c2,e2);
xadd(c2,c3,c2);
xln(c2,c2);
xln(e2,e2);
xsub(c2,e2,e2);
xdiv2s(e2)
end
else begin
inc(usrdigits,3);
move(ar0d5,c5,9);
np:=0;
while xgt(c2,c5) do begin
xmul(c2,c2,c4);
xsub(c3,c4,c4);
xsqrt(c4,c4);
xaddb(c4,1);
xdiv(c2,c4,c2);
inc(np)
end;
xtox(c2,e2);
xmul(c2,c2,c4);
uf:=1;
repeat
inc(uf,2);
recip;
xmul(c2,c4,c2);
xmul(c2,rc,c5);
xadd(e2,c5,e2);
until (c5.n=1) and (c5.c[1]=0);
while np>0 do begin
xmul2s(e2);
dec(np)
end;
dec(usrdigits,3);
xapp(e2,e2,usrdigits);
e2.s:=e1.s
end
end;

{* arcoth(x) = 0.5*(ln(x+1)-ln(x-1)) *}

procedure xacoth(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
move(arm1,c2,8);
move(ar1,c3,8);
if xgte(e2,c2) and xlte(e2,c3) then begin
xstatus:=33;
move(ar0,e2,8);
exit
end;
e2.s:=1;
xsub(e2,c3,c2);
xadd(e2,c3,e2);
xln(c2,c2);
xln(e2,e2);
xsub(e2,c2,e2);
xdiv2s(e2);
if (e2.c[1]>0) or (e2.n>1) then e2.s:=e1.s
end;

{* arsech(x) = ln((sqrt(1-x*x)+1)/x) *}

procedure xasech(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
move(ar0,c2,8);
move(ar1,c3,8);
if (not xgt(e2,c2)) or xgt(e2,c3) then begin
xstatus:=34;
move(ar0,e2,8);
exit
end;
xmul(e2,e2,c2);
c2.s:=c2.s xor 1;
xadd(c3,c2,c2);
xsqrt(c2,c2);
xadd(c2,c3,c2);
xdiv(c2,e2,e2);
xln(e2,e2)
end;

{* arcsch(x) = ln(1/x+sqrt(1/x^2+1)) *}

procedure xacsch(e1: extnum; var e2: extnum);
begin
xtox(e1,e2);
xelz(e2);
if (e2.c[1]=0) and (e2.n=1) then begin
xstatus:=35;
move(ar0,e2,8);
exit
end;
move(ar1,c2,8);
e2.s:=1;
xdiv(c2,e2,e2);
xmul(e2,e2,c3);
xadd(c3,c2,c3);
xsqrt(c3,c3);
xadd(c3,e2,c2);
xln(c2,e2);
e2.s:=e1.s
end;

{* bitwise not *}

procedure notm(e1: extnum; var e2: extnum);
begin
if (e1.c[1]=0) and (e1.i=1) then begin
move(ar1,e2,8);
exit
end;
extobuf(e1,bf);
ix:=bf[0];
while (bf[ix]=0) and (ix>1) do dec(ix);
ml:=bf[ix];
pl:=0;
zl:=0;
repeat
if ml and 1=0 then inc(pl,pow2[zl]);
inc(zl);
ml:=ml shr 1;
until ml=0;
bf[ix]:=pl;
if ix>1 then
for jv:=1 to ix-1 do bf[jv]:=not bf[jv];
while (bf[ix]=0) and (ix>1) do dec(ix);
bf[0]:=ix;
buftoex(bf,e2)
end;

{* bitwise and *}

procedure andm(e1,e2: extnum; var e3: extnum);
begin
extobuf(e1,bf1);
extobuf(e2,bf2);
ix:=bf1[0];
kx:=bf2[0];
if ix<kx then begin
for jv:=ix+1 to kx do bf1[jv]:=0;
iq:=kx
end
else if kx<ix then begin
for jv:=kx+1 to ix do bf2[jv]:=0;
iq:=ix
end
else iq:=ix;
for jv:=1 to iq do bf1[jv]:=bf1[jv] and bf2[jv];
while (bf1[iq]=0) and (iq>1) do dec(iq);
bf1[0]:=iq;
buftoex(bf1,e3)
end;

{* bitwise or *}

procedure orm(e1,e2: extnum; var e3: extnum);
begin
extobuf(e1,bf1);
extobuf(e2,bf2);
ix:=bf1[0];
kx:=bf2[0];
if ix<kx then begin
for jv:=ix+1 to kx do bf1[jv]:=0;
iq:=kx
end
else if kx<ix then begin
for jv:=kx+1 to ix do bf2[jv]:=0;
iq:=ix
end
else iq:=ix;
for jv:=1 to iq do bf1[jv]:=bf1[jv] or bf2[jv];
while (bf1[iq]=0) and (iq>1) do dec(iq);
bf1[0]:=iq;
buftoex(bf1,e3)
end;

{* bitwise xor *}

procedure xorm(e1,e2: extnum; var e3: extnum);
begin
extobuf(e1,bf1);
extobuf(e2,bf2);
ix:=bf1[0];
kx:=bf2[0];
if ix<kx then begin
for jv:=ix+1 to kx do bf1[jv]:=0;
iq:=kx
end
else if kx<ix then begin
for jv:=kx+1 to ix do bf2[jv]:=0;
iq:=ix
end
else iq:=ix;
for jv:=1 to iq do bf1[jv]:=bf1[jv] xor bf2[jv];
while (bf1[iq]=0) and (iq>1) do dec(iq);
bf1[0]:=iq;
buftoex(bf1,e3)
end;

{* special functions *}

{Error function. Domain: all real numbers. Codomain: ]-1,+1[.
Calculated by means of the positive series:
(2/sqrt(pi))*exp(-x^2)*(x+2*x^3/1*3+(2^2)*x^5/1*3*5+(2^3)*x^7/1*3*5*7+...)
If overflow, returns -1 if x<0 or +1 if x>0}

procedure xerf(e1: extnum; var e2: extnum);
label rec0,quit;
begin
xstatus:=0;
xtox(e1,e2);
xelz(e2);
if (e2.c[1]=0) and (e2.n=1) then exit;
e2.s:=1;
xmul(e2,e2,a3);
xexp(a3,c8);
if xstatus=19 then goto quit;
pitox(a7);
xsqrt(a7,a7);
xdiv2s(a7);
xmul(a7,c8,a7);
uz:=1;
xmul2s(a3);
xtox(e2,a6);
rec0:
inc(uz,2);
kz:=1;
rz:=10;
while uz>rz do begin
rz:=rz*10;
inc(kz);
rd.c[kz]:=0
end;
while (kz<usrdigits) and (rz>0) do begin
inc(kz);
vz:=rz div uz;
rz:=(rz-vz*uz)*10;
rd.c[kz]:=vz
end;
rd.n:=kz; rd.d:=kz-1;
xmul(a3,rd,a2);
if xstatus>0 then goto quit;
xmul(a6,a2,a6);
if xstatus>0 then goto quit;
xadd(e2,a6,e2);
if (a6.n>1) or (a6.c[1]>0) then goto rec0;
quit:
if xstatus>0 then begin
xstatus:=0;
move(ar1,e2,8)
end
else xdiv(e2,a7,e2);
e2.s:=e1.s
end;

{Inverse error function. Domain: ]-1,+1[. Codomain: all real numbers.
Calculated by means of the convergence to zero of xerf(x)-z where z is the
argument. Possible errors: xstatus=50 if x<=-1 or x>=+1 (out of domain);
xstatus=51 if overflow}

procedure set_tolerance;
begin
erfprec:=(usrdigits-10) div 2;
fillbyte(vp.c,erfprec,0);
vp.c[erfprec+1]:=1;
vp.n:=erfprec+1; vp.i:=1; vp.d:=erfprec; vp.s:=1
end;

procedure xerfinv(e1: extnum; var e2: extnum);
label rec2,rec3;

procedure call_ierf;
Label rec1;
begin
xmul(e2,e2,a3);
xexp(a3,c8);
if xstatus>0 then exit;
xmul(v7,c8,a7);
uz:=1;
xtox(e2,ez);
xtox(e2,a6);
xmul2s(a3);
rec1:
inc(uz,2);
kz:=1; rz:=10;
while uz>rz do begin
rz:=rz*10; inc(kz); rd.c[kz]:=0
end;
while (kz<usrdigits) and (rz>0) do begin
inc(kz);
vz:=rz div uz;
rz:=(rz-vz*uz)*10;
rd.c[kz]:=vz
end;
rd.n:=kz; rd.d:=kz-1;
xmul(a3,rd,a2);
if xstatus>0 then exit;
xmul(a6,a2,a6);
if xstatus>0 then exit;
xadd(ez,a6,ez);
if (a6.n>1) or (a6.c[1]>0) then goto rec1;
xdiv(ez,a7,v4)
end;

begin
xtox(e1,v2);
xelz(v2);
v2.s:=1;
xstatus:=0;
if (v2.c[1]=0) and (v2.n=1) then begin
move(ar0,e2,8);
exit
end;
if v2.c[1]>0 then begin
xstatus:=50;
move(ar0,e2,8);
exit
end;
set_tolerance;
xtox(v2,a3);
pitox(a2);
xsqrt(a2,a2);
move(ar1,a4,8);
xsub(a4,a3,a3);
xmul(a2,a3,a3);
xln(a3,v5);
v5.s:=1;
xtox(v5,a3);
xmul(v5,v5,v6);
xmul(v6,v5,v7);
xmul(v7,v5,v8);
xln(v5,a7);
xdiv2s(a7);
xmul(a7,a7,v2);
xmul(v2,a7,v3);
xmul(v3,a7,v4);
xsub(a3,a7,a3);
xsub(a7,a4,a5);
xdiv2s(a5);
xdiv(a5,v5,a5);
xadd(a3,a5,a3);
xsqrt(a3,c2);
xmulb(a7,3,a5);
xsub(v2,a5,a5);
move(ar3d5,a6,9);
xadd(a5,a6,a5);
xmulb(v6,4,a6);
xdiv(a5,a6,a5);
xadd(a3,a5,a3);
xsqrt(a3,c3);
xdivb(v3,3,a5);
move(ar1d75,a4,10);
xmul(v2,a4,a6);
xsub(a5,a6,a5);
move(ar4d25,a4,10);
xmul(a7,a4,a6);
xadd(a5,a6,a5);
move(ar107,a4,10);
move(ar24,a6,9);
xdiv(a4,a6,a2);
xsub(a5,a2,a5);
xmul2s(v7);
xdiv(a5,v7,a5);
xadd(a3,a5,a3);
xsqrt(a3,c4);
xdiv2s(v4);
a6.c[2]:=3;
xmul(v3,a6,a5);
xdivbs(a5,6);
xsub(v4,a5,v4);
move(ar14d5,a6,10);
xmul(v2,a6,a5);
xadd(v4,a5,v4);
move(ar31,a4,9);
xmul(a7,a4,a5);
xsub(v4,a5,v4);
move(ar1489,a6,11);
move(ar48,a4,9);
xdiv(a6,a4,a6);
xadd(v4,a6,v4);
xmulbs(v8,4);
xdiv(v4,v8,v4);
xadd(a3,v4,a3);
xsqrt(a3,c5);
xsub(c3,c2,a3);
xsub(c5,c4,a4);
if xgt(a3,a4) then xtox(c4,e2) else xtox(c2,e2);
pitox(v7);
xsqrt(v7,v7);
xdiv2s(v7);
xtox(e1,v2);
v2.s:=1;
move(ar1,v8,8);
xtoe(e2,xz);
if (xz<4.593999445394043) or (xz>12.65799999994681) then begin
move(ar2,v3,8);
move(ar3,a4,8);
xdiv(v3,a4,v3);
move(ar0d5,a5,9);
rec2:
call_ierf;
if xstatus>0 then begin
xstatus:=51;
move(ar0,e2,8);
exit
end;
xsub(v2,v4,v4);
xmul(v4,v7,v4);
xmul(e2,e2,v5);
xexp(v5,v6);
if xstatus=19 then begin
xstatus:=51;
move(ar0,e2,8);
exit
end;
xmul(v4,v6,c8);
xmul(v3,c8,v6);
xadd(e2,v6,v6);
xmul(v6,v6,v6);
xsub(v5,v6,v6);
xexp(v6,v6);
if xstatus=19 then begin
xstatus:=51;
move(ar0,e2,8);
exit
end;
xmul(a4,v6,v5);
xsub(v5,v8,v5);
xdiv(v8,v5,v5);
xadd(v5,a5,v5);
xmul(c8,v5,v5);
xadd(e2,v5,e2);
if xgt(v5,vp) then goto rec2
end
else begin
rec3:
call_ierf;
if xstatus>0 then begin
xstatus:=51;
move(ar0,e2,8);
exit
end;
xsub(v2,v4,v4);
xmul(e2,e2,v6);
xexp(v6,v6);
if xstatus=19 then begin
xstatus:=51;
move(ar0,e2,8);
exit
end;
xmul(v4,v6,v4);
xmul(v4,v7,v4);
xmul(e2,v4,v6);
xadd(v6,v8,v6);
xmul(v6,v4,v6);
xadd(e2,v6,e2);
if xgt(v6,vp) then goto rec3
end;
e2.s:=e1.s
end;

{Complementary error function: xerfc(x)=1-xerf(x).
Domain: all real numbers. Codomain: ]0,2[. Returns 0 or 2 if overflow}

procedure xerfc(e1: extnum; var e2: extnum);
begin
xerf(e1,e2);
move(ar1,c3,8);
xsub(c3,e2,e2)
end;

{Inverse complementary error function: xerfcinv(x)=xerfinv(1-x)
Domain: ]0,2[. Codomain: all real numbers.
Possible errors: xstatus=52 if x<=0 or x>=2 (out of domain); xstatus=53
if overflow}

procedure xerfcinv(e1: extnum; var e2: extnum);
begin
move(ar0,c3,8);
move(ar2,c4,8);
if xlte(e1,c3) or xgte(e1,c4) then begin
xstatus:=52;
move(ar0,e2,8);
exit
end;
c4.c[1]:=1;
xsub(c4,e1,e2);
xerfinv(e2,e2);
if xstatus=51 then xstatus:=53
end;

{Cumulative normal distribution function, (1/sqrt(2*pi)) multiplied by
the integral from 0 to x of exp(-t^2/2)dt. xphi(x) = 0.5*xerf(x/sqrt(2)).
Domain: [0,+inf[. Codomain: [0,1/2[. Possible errors: xstatus=54 if x<0.
Returns 1/2 if overflow}

procedure xphi(e1: extnum; var e2: extnum);
begin
if e1.s=0 then begin
xstatus:=54;
move(ar0,e2,8);
exit
end;
move(ar2,c4,8);
xsqrt(c4,c4);
xdiv(e1,c4,e2);
xerf(e2,e2);
xdiv2s(e2)
end;

{Inverse cumulative normal distribution function:
xiphi(x)=sqrt(2)*xerfinv(2*x). Domain: [0,1/2[. Codomain: [0,+inf[.
Possible errors: xstatus=55 if x<0 or x>=1/2 (out of domain); xstatus=56
if overflow}

procedure xiphi(e1: extnum; var e2: extnum);
begin
move(ar0d5,c4,9);
if (e1.s=0) or xgte(e1,c4) then begin
xstatus:=55;
move(ar0,e2,8);
exit
end;
xtox(e1,e2);
xmul2s(e2);
xerfinv(e2,e2);
if xstatus=51 then begin
xstatus:=56;
exit
end;
move(ar2,c4,8);
xsqrt(c4,c4);
xmul(c4,e2,e2)
end;

{Lower tail cumulative probability of the standard normal distribution:
(1/sqrt(2*pi)) multiplied by the integral from -inf to x of exp(t^2/2)dt.
xlnor(x)=0.5*(1+xerf(x/sqrt(2))). Domain: all real numbers. Codomain: ]0,1[.
Returns 0 or 1 if overflow}

procedure xlnor(e1: extnum; var e2: extnum);
begin
move(ar2,c4,8);
xsqrt(c4,c4);
xdiv(e1,c4,e2);
xerf(e2,e2);
move(ar1,c4,8);
xadd(e2,c4,e2);
xdiv2s(e2)
end;

{Inverse lower tail cumulative probability of the standard normal distribution:
xilnor(x)=sqrt(2)*xerfinv(2*x-1). Domain: ]0,1[. Codomain: all real numbers.
Possible errors: xstatus=57 if x<=0 or x>=1 (out of domain); xstatus=58
if overflow}

procedure xilnor(e1: extnum; var e2: extnum);
begin
move(ar0,c3,8);
move(ar1,c4,8);
if xlte(e1,c3) or xgte(e1,c4) then begin
xstatus:=57;
move(ar0,e2,8);
exit
end;
xtox(e1,e2);
xmul2s(e2);
xsub(e2,c4,e2);
xerfinv(e2,e2);
if xstatus=51 then begin
xstatus:=58;
exit
end;
move(ar2,c4,8);
xsqrt(c4,c4);
xmul(c4,e2,e2)
end;

{Upper tail cumulative probability of the standard normal distribution:
(1/sqrt(2*pi)) multiplied by the integral from x to +inf of exp(-t^2/2)dt.
xunor(x)=0.5*(1-xerf(x/sqrt(2))). Domain: all real numbers. Codomain: ]0,1[.
Returns 0 or 1 if overflow}

procedure xunor(e1: extnum; var e2: extnum);
begin
move(ar2,c4,8);
xsqrt(c4,c4);
xdiv(e1,c4,e2);
xerf(e2,e2);
move(ar1,c4,8);
xsub(c4,e2,e2);
xdiv2s(e2)
end;

{Inverse upper tail cumulative probability of the standard normal distribution:
xiunor(x)=sqrt(2)*xerfinv(1-2*x). Domain: ]0,1[. Codomain: all real numbers.
Possible errors: xstatus=59 if x<=0 or x>=1 (out of domain); xstatus=60
if overflow}

procedure xiunor(e1: extnum; var e2: extnum);
begin
move(ar0,c3,8);
move(ar1,c4,8);
if xlte(e1,c3) or xgte(e1,c4) then begin
xstatus:=59;
move(ar0,e2,8);
exit
end;
xtox(e1,e2);
xmul2s(e2);
xsub(c4,e2,e2);
xerfinv(e2,e2);
if xstatus=51 then begin
xstatus:=60;
exit
end;
move(ar2,c4,8);
xsqrt(c4,c4);
xmul(c4,e2,e2)
end;

BEGIN
xsetn(500);
xstatus:=0;
rc.c[1]:=0;
rc.i:=1;
rc.s:=1;
rd.c[1]:=0;
rd.i:=1;
rd.s:=1;
subflag:=false
END.
